@@ -28,11 +28,19 @@ ANSI_RESET = '\033[0m'
 
 
 def colorme(bval, colormap, colorkey):
+    if colormap is NO_COLOR_MAP:
+        return bval
     if colormap is None:
         colormap = DEFAULT_VALUE_COLORS
     return FormattedValue(bval, colormap[colorkey] + bval + colormap['reset'])
 
 
+def get_str(val):
+    if isinstance(val, FormattedValue):
+        return val.strval
+    return val
+
+
 class FormattedValue:
 
     def __init__(self, strval, coloredval=None, displaywidth=None):
@@ -91,18 +99,22 @@ class FormattedValue:
         """
         return self.coloredval + self._pad(width, fill)
 
+
 DEFAULT_VALUE_COLORS = dict(
     default=YELLOW,
     text=YELLOW,
     error=RED,
     blob=DARK_MAGENTA,
     timestamp=GREEN,
+    date=GREEN,
+    time=GREEN,
     int=GREEN,
     float=GREEN,
     decimal=GREEN,
     inet=GREEN,
     boolean=GREEN,
     uuid=GREEN,
+    duration=GREEN,
     collection=BLUE,
     reset=ANSI_RESET,
 )
@@ -112,3 +124,5 @@ COLUMN_NAME_COLORS = defaultdict(lambda: MAGENTA,
                                  blob=DARK_MAGENTA,
                                  reset=ANSI_RESET,
                                  )
+
+NO_COLOR_MAP = dict()
