@@ -21,6 +21,7 @@ import java.io.IOException;
 import java.nio.ByteBuffer;
 
 import com.google.common.base.Objects;
+import com.google.common.collect.Range;
 
 import org.apache.cassandra.cql3.ColumnSpecification;
 import org.apache.cassandra.cql3.QueryOptions;
@@ -43,12 +44,19 @@ import org.apache.cassandra.utils.ByteBufferUtil;
  */
 abstract class ElementsSelector extends Selector
 {
+    /**
+     * An empty collection is composed of an int size of zero.
+     */
+    private static final ByteBuffer EMPTY_FROZEN_COLLECTION = ByteBufferUtil.bytes(0);
+
     protected final Selector selected;
+    protected final CollectionType<?> type;
 
     protected ElementsSelector(Kind kind,Selector selected)
     {
         super(kind);
         this.selected = selected;
+        this.type = getCollectionType(selected);
     }
 
     private static boolean isUnset(ByteBuffer bb)
@@ -237,9 +245,14 @@ abstract class ElementsSelector extends Selector
 
     protected abstract ByteBuffer extractSelection(ByteBuffer collection);
 
-    public void addInput(ProtocolVersion protocolVersion, InputRow input)
+    public void addInput(InputRow input)
     {
-        selected.addInput(protocolVersion, input);
+        selected.addInput(input);
+    }
+
+    protected Range<Integer> getIndexRange(ByteBuffer output, ByteBuffer fromKey, ByteBuffer toKey)
+    {
+        return type.getSerializer().getIndexesRangeFromSerialized(output, fromKey, toKey, keyType(type));
     }
 
     public void reset()
@@ -266,13 +279,11 @@ abstract class ElementsSelector extends Selector
             }
         };
 
-        private final CollectionType<?> type;
         private final ByteBuffer key;
 
         private ElementSelector(Selector selected, ByteBuffer key)
         {
             super(Kind.ELEMENT_SELECTOR, selected);
-            this.type = getCollectionType(selected);
             this.key = key;
         }
 
@@ -294,6 +305,31 @@ abstract class ElementsSelector extends Selector
             return type.getSerializer().getSerializedValue(collection, key, keyType(type));
         }
 
+        protected int getElementIndex(ProtocolVersion protocolVersion, ByteBuffer key)
+        {
+            ByteBuffer output = selected.getOutput(protocolVersion);
+            return output == null ? -1 : type.getSerializer().getIndexFromSerialized(output, key, keyType(type));
+        }
+
+        @Override
+        protected ColumnTimestamps getWritetimes(ProtocolVersion protocolVersion)
+        {
+            return getElementTimestamps(protocolVersion, selected.getWritetimes(protocolVersion));
+        }
+
+        @Override
+        protected ColumnTimestamps getTTLs(ProtocolVersion protocolVersion)
+        {
+            return getElementTimestamps(protocolVersion, selected.getTTLs(protocolVersion));
+        }
+
+        private ColumnTimestamps getElementTimestamps(ProtocolVersion protocolVersion,
+                                                      ColumnTimestamps timestamps)
+        {
+            int index = getElementIndex(protocolVersion, key);
+            return index == -1 ? ColumnTimestamps.NO_TIMESTAMP : timestamps.get(index);
+        }
+
         public AbstractType<?> getType()
         {
             return valueType(type);
@@ -358,8 +394,6 @@ abstract class ElementsSelector extends Selector
             }
         };
 
-        private final CollectionType<?> type;
-
         // Note that neither from nor to can be null, but they can both be ByteBufferUtil.UNSET_BYTE_BUFFER to represent no particular bound
         private final ByteBuffer from;
         private final ByteBuffer to;
@@ -368,7 +402,6 @@ abstract class ElementsSelector extends Selector
         {
             super(Kind.SLICE_SELECTOR, selected);
             assert from != null && to != null : "We can have unset buffers, but not nulls";
-            this.type = getCollectionType(selected);
             this.from = from;
             this.to = to;
         }
@@ -391,6 +424,36 @@ abstract class ElementsSelector extends Selector
             return type.getSerializer().getSliceFromSerialized(collection, from, to, type.nameComparator(), type.isFrozenCollection());
         }
 
+        @Override
+        protected ColumnTimestamps getWritetimes(ProtocolVersion protocolVersion)
+        {
+            return getTimestampsSlice(protocolVersion, selected.getWritetimes(protocolVersion));
+        }
+
+        @Override
+        protected ColumnTimestamps getTTLs(ProtocolVersion protocolVersion)
+        {
+            return getTimestampsSlice(protocolVersion, selected.getTTLs(protocolVersion));
+        }
+
+        protected ColumnTimestamps getTimestampsSlice(ProtocolVersion protocolVersion, ColumnTimestamps timestamps)
+        {
+            ByteBuffer output = selected.getOutput(protocolVersion);
+            return (output == null || isCollectionEmpty(output))
+                   ? ColumnTimestamps.NO_TIMESTAMP
+                   : timestamps.slice(getIndexRange(output, from, to) );
+        }
+
+        /**
+         * Checks if the collection is empty. Only frozen collection can be empty.
+         * @param output the serialized collection
+         * @return {@code true} if the collection is empty {@code false} otherwise.
+         */
+        private boolean isCollectionEmpty(ByteBuffer output)
+        {
+            return EMPTY_FROZEN_COLLECTION.equals(output);
+        }
+
         public AbstractType<?> getType()
         {
             return type;
