@@ -153,6 +153,7 @@ public abstract class AlterTypeStatement extends AlterSchemaStatement
             }
 
             Guardrails.fieldsPerUDT.guard(userType.size() + 1, userType.getNameAsString(), false, state);
+            type.validate(state, "Field " + fieldName);
 
             List<FieldIdentifier> fieldNames = new ArrayList<>(userType.fieldNames()); fieldNames.add(fieldName);
             List<AbstractType<?>> fieldTypes = new ArrayList<>(userType.fieldTypes()); fieldTypes.add(fieldType);
@@ -185,7 +186,7 @@ public abstract class AlterTypeStatement extends AlterSchemaStatement
         UserType apply(KeyspaceMetadata keyspace, UserType userType)
         {
             List<String> dependentAggregates =
-                keyspace.functions
+                keyspace.userFunctions
                         .udas()
                         .filter(uda -> null != uda.initialCondition() && uda.stateType().referencesUserType(userType.name))
                         .map(uda -> uda.name().toString())
