@@ -36,6 +36,7 @@ import org.apache.cassandra.db.guardrails.Guardrails;
 import org.apache.cassandra.db.marshal.MapType;
 import org.apache.cassandra.db.marshal.AbstractType;
 import org.apache.cassandra.exceptions.InvalidRequestException;
+import org.apache.cassandra.index.internal.CassandraIndex;
 import org.apache.cassandra.index.sasi.SASIIndex;
 import org.apache.cassandra.schema.*;
 import org.apache.cassandra.schema.Keyspaces.KeyspacesDiff;
@@ -49,6 +50,37 @@ import static com.google.common.collect.Iterables.tryFind;
 
 public final class CreateIndexStatement extends AlterSchemaStatement
 {
+    public static final String SASI_INDEX_DISABLED = "SASI indexes are disabled. Enable in cassandra.yaml to use.";
+    public static final String KEYSPACE_DOES_NOT_EXIST = "Keyspace '%s' doesn't exist";
+    public static final String TABLE_DOES_NOT_EXIST = "Table '%s' doesn't exist";
+    public static final String COUNTER_TABLES_NOT_SUPPORTED = "Secondary indexes on counter tables aren't supported";
+    public static final String MATERIALIZED_VIEWS_NOT_SUPPORTED = "Secondary indexes on materialized views aren't supported";
+    public static final String TRANSIENTLY_REPLICATED_KEYSPACE_NOT_SUPPORTED = "Secondary indexes are not supported on transiently replicated keyspaces";
+    public static final String CUSTOM_CREATE_WITHOUT_COLUMN = "Only CUSTOM indexes can be created without specifying a target column";
+    public static final String CUSTOM_MULTIPLE_COLUMNS = "Only CUSTOM indexes support multiple columns";
+    public static final String DUPLICATE_TARGET_COLUMN = "Duplicate column '%s' in index target list";
+    public static final String COLUMN_DOES_NOT_EXIST = "Column '%s' doesn't exist";
+    public static final String INVALID_CUSTOM_INDEX_TARGET = "Column '%s' is longer than the permissible name length of %d characters or" +
+                                                             " contains non-alphanumeric-underscore characters";
+    public static final String COLLECTIONS_WITH_DURATIONS_NOT_SUPPORTED = "Secondary indexes are not supported on collections containing durations";
+    public static final String TUPLES_WITH_DURATIONS_NOT_SUPPORTED = "Secondary indexes are not supported on tuples containing durations";
+    public static final String DURATIONS_NOT_SUPPORTED = "Secondary indexes are not supported on duration columns";
+    public static final String UDTS_WITH_DURATIONS_NOT_SUPPORTED = "Secondary indexes are not supported on UDTs containing durations";
+    public static final String PRIMARY_KEY_IN_COMPACT_STORAGE = "Secondary indexes are not supported on PRIMARY KEY columns in COMPACT STORAGE tables";
+    public static final String COMPACT_COLUMN_IN_COMPACT_STORAGE = "Secondary indexes are not supported on compact value column of COMPACT STORAGE tables";
+    public static final String ONLY_PARTITION_KEY = "Cannot create secondary index on the only partition key column %s";
+    public static final String CREATE_ON_FROZEN_COLUMN = "Cannot create %s() index on frozen column %s. Frozen collections are immutable and must be fully " +
+                                                         "indexed by using the 'full(%s)' modifier";
+    public static final String FULL_ON_FROZEN_COLLECTIONS = "full() indexes can only be created on frozen collections";
+    public static final String NON_COLLECTION_SIMPLE_INDEX = "Cannot create %s() index on %s. Non-collection columns only support simple indexes";
+    public static final String CREATE_WITH_NON_MAP_TYPE = "Cannot create index on %s of column %s with non-map type";
+    public static final String CREATE_ON_NON_FROZEN_UDT = "Cannot create index on non-frozen UDT column %s";
+    public static final String INDEX_ALREADY_EXISTS = "Index '%s' already exists";
+    public static final String INDEX_DUPLICATE_OF_EXISTING = "Index %s is a duplicate of existing index %s";
+    public static final String KEYSPACE_DOES_NOT_MATCH_TABLE = "Keyspace name '%s' doesn't match table name '%s'";
+    public static final String KEYSPACE_DOES_NOT_MATCH_INDEX = "Keyspace name '%s' doesn't match index name '%s'";
+    public static final String MUST_SPECIFY_INDEX_IMPLEMENTATION = "Must specify index implementation via USING";
+
     private final String indexName;
     private final String tableName;
     private final List<IndexTarget.Raw> rawIndexTargets;
@@ -88,32 +120,32 @@ public final class CreateIndexStatement extends AlterSchemaStatement
         Guardrails.createSecondaryIndexesEnabled.ensureEnabled("Creating secondary indexes", state);
 
         if (attrs.isCustom && attrs.customClass.equals(SASIIndex.class.getName()) && !DatabaseDescriptor.getSASIIndexesEnabled())
-            throw new InvalidRequestException("SASI indexes are disabled. Enable in cassandra.yaml to use.");
+            throw new InvalidRequestException(SASI_INDEX_DISABLED);
 
         KeyspaceMetadata keyspace = schema.getNullable(keyspaceName);
         if (null == keyspace)
-            throw ire("Keyspace '%s' doesn't exist", keyspaceName);
+            throw ire(KEYSPACE_DOES_NOT_EXIST, keyspaceName);
 
         TableMetadata table = keyspace.getTableOrViewNullable(tableName);
         if (null == table)
-            throw ire("Table '%s' doesn't exist", tableName);
+            throw ire(TABLE_DOES_NOT_EXIST, tableName);
 
         if (null != indexName && keyspace.hasIndex(indexName))
         {
             if (ifNotExists)
                 return schema;
 
-            throw ire("Index '%s' already exists", indexName);
+            throw ire(INDEX_ALREADY_EXISTS, indexName);
         }
 
         if (table.isCounter())
-            throw ire("Secondary indexes on counter tables aren't supported");
+            throw ire(COUNTER_TABLES_NOT_SUPPORTED);
 
         if (table.isView())
-            throw ire("Secondary indexes on materialized views aren't supported");
+            throw ire(MATERIALIZED_VIEWS_NOT_SUPPORTED);
 
         if (Keyspace.open(table.keyspace).getReplicationStrategy().hasTransientReplicas())
-            throw new InvalidRequestException("Secondary indexes are not supported on transiently replicated keyspaces");
+            throw new InvalidRequestException(TRANSIENTLY_REPLICATED_KEYSPACE_NOT_SUPPORTED);
 
         // guardrails to limit number of secondary indexes per table.
         Guardrails.secondaryIndexesPerTable.guard(table.indexes.size() + 1,
@@ -126,25 +158,25 @@ public final class CreateIndexStatement extends AlterSchemaStatement
         List<IndexTarget> indexTargets = Lists.newArrayList(transform(rawIndexTargets, t -> t.prepare(table)));
 
         if (indexTargets.isEmpty() && !attrs.isCustom)
-            throw ire("Only CUSTOM indexes can be created without specifying a target column");
+            throw ire(CUSTOM_CREATE_WITHOUT_COLUMN);
 
         if (indexTargets.size() > 1)
         {
             if (!attrs.isCustom)
-                throw ire("Only CUSTOM indexes support multiple columns");
+                throw ire(CUSTOM_MULTIPLE_COLUMNS);
 
             Set<ColumnIdentifier> columns = new HashSet<>();
             for (IndexTarget target : indexTargets)
                 if (!columns.add(target.column))
-                    throw ire("Duplicate column '%s' in index target list", target.column);
+                    throw ire(DUPLICATE_TARGET_COLUMN, target.column);
         }
 
-        indexTargets.forEach(t -> validateIndexTarget(table, t));
+        IndexMetadata.Kind kind = attrs.isCustom ? IndexMetadata.Kind.CUSTOM : IndexMetadata.Kind.COMPOSITES;
+
+        indexTargets.forEach(t -> validateIndexTarget(table, kind, t));
 
         String name = null == indexName ? generateIndexName(keyspace, indexTargets) : indexName;
 
-        IndexMetadata.Kind kind = attrs.isCustom ? IndexMetadata.Kind.CUSTOM : IndexMetadata.Kind.COMPOSITES;
-
         Map<String, String> options = attrs.isCustom ? attrs.getOptions() : Collections.emptyMap();
 
         IndexMetadata index = IndexMetadata.fromIndexTargets(indexTargets, name, kind, options);
@@ -156,7 +188,7 @@ public final class CreateIndexStatement extends AlterSchemaStatement
             if (ifNotExists)
                 return schema;
 
-            throw ire("Index %s is a duplicate of existing index %s", index.name, equalIndex.name);
+            throw ire(INDEX_DUPLICATE_OF_EXISTING, index.name, equalIndex.name);
         }
 
         TableMetadata newTable = table.withSwapped(table.indexes.with(index));
@@ -174,56 +206,58 @@ public final class CreateIndexStatement extends AlterSchemaStatement
         return ImmutableSet.of();
     }
 
-    private void validateIndexTarget(TableMetadata table, IndexTarget target)
+    private void validateIndexTarget(TableMetadata table, IndexMetadata.Kind kind, IndexTarget target)
     {
         ColumnMetadata column = table.getColumn(target.column);
 
         if (null == column)
-            throw ire("Column '%s' doesn't exist", target.column);
+            throw ire(COLUMN_DOES_NOT_EXIST, target.column);
 
         AbstractType<?> baseType = column.type.unwrap();
 
+        if ((kind == IndexMetadata.Kind.CUSTOM) && !SchemaConstants.isValidName(target.column.toString()))
+            throw ire(INVALID_CUSTOM_INDEX_TARGET, target.column, SchemaConstants.NAME_LENGTH);
+
         if (column.type.referencesDuration())
         {
             if (column.type.isCollection())
-                throw ire("Secondary indexes are not supported on collections containing durations");
+                throw ire(COLLECTIONS_WITH_DURATIONS_NOT_SUPPORTED);
 
             if (column.type.isTuple())
-                throw ire("Secondary indexes are not supported on tuples containing durations");
+                throw ire(TUPLES_WITH_DURATIONS_NOT_SUPPORTED);
 
             if (column.type.isUDT())
-                throw  ire("Secondary indexes are not supported on UDTs containing durations");
+                throw  ire(UDTS_WITH_DURATIONS_NOT_SUPPORTED);
 
-            throw ire("Secondary indexes are not supported on duration columns");
+            throw ire(DURATIONS_NOT_SUPPORTED);
         }
 
         if (table.isCompactTable())
         {
             TableMetadata.CompactTableMetadata compactTable = (TableMetadata.CompactTableMetadata) table;
             if (column.isPrimaryKeyColumn())
-                throw new InvalidRequestException("Secondary indexes are not supported on PRIMARY KEY columns in COMPACT STORAGE tables");
+                throw new InvalidRequestException(PRIMARY_KEY_IN_COMPACT_STORAGE);
             if (compactTable.compactValueColumn.equals(column))
-                throw new InvalidRequestException("Secondary indexes are not supported on compact value column of COMPACT STORAGE tables");
+                throw new InvalidRequestException(COMPACT_COLUMN_IN_COMPACT_STORAGE);
         }
 
         if (column.isPartitionKey() && table.partitionKeyColumns().size() == 1)
-            throw ire("Cannot create secondary index on the only partition key column %s", column);
+            throw ire(ONLY_PARTITION_KEY, column);
 
         if (baseType.isFrozenCollection() && target.type != Type.FULL)
-            throw ire("Cannot create %s() index on frozen column %s. Frozen collections are immutable and must be fully " +
-                      "indexed by using the 'full(%s)' modifier", target.type, column, column);
+            throw ire(CREATE_ON_FROZEN_COLUMN, target.type, column, column.name.toCQLString());
 
         if (!baseType.isFrozenCollection() && target.type == Type.FULL)
-            throw ire("full() indexes can only be created on frozen collections");
+            throw ire(FULL_ON_FROZEN_COLLECTIONS);
 
         if (!baseType.isCollection() && target.type != Type.SIMPLE)
-            throw ire("Cannot create %s() index on %s. Non-collection columns only support simple indexes", target.type, column);
+            throw ire(NON_COLLECTION_SIMPLE_INDEX, target.type, column);
 
-        if (!(baseType instanceof MapType && column.type.isMultiCell()) && (target.type == Type.KEYS || target.type == Type.KEYS_AND_VALUES))
-            throw ire("Cannot create index on %s of column %s with non-map type", target.type, column);
+        if (!(baseType instanceof MapType && baseType.isMultiCell()) && (target.type == Type.KEYS || target.type == Type.KEYS_AND_VALUES))
+            throw ire(CREATE_WITH_NON_MAP_TYPE, target.type, column);
 
         if (column.type.isUDT() && column.type.isMultiCell())
-            throw ire("Cannot create index on non-frozen UDT column %s", column);
+            throw ire(CREATE_ON_NON_FROZEN_UDT, column);
     }
 
     private String generateIndexName(KeyspaceMetadata keyspace, List<IndexTarget> targets)
@@ -283,10 +317,31 @@ public final class CreateIndexStatement extends AlterSchemaStatement
                                 : indexName.hasKeyspace() ? indexName.getKeyspace() : state.getKeyspace();
 
             if (tableName.hasKeyspace() && !keyspaceName.equals(tableName.getKeyspace()))
-                throw ire("Keyspace name '%s' doesn't match table name '%s'", keyspaceName, tableName);
+                throw ire(KEYSPACE_DOES_NOT_MATCH_TABLE, keyspaceName, tableName);
 
             if (indexName.hasKeyspace() && !keyspaceName.equals(indexName.getKeyspace()))
-                throw ire("Keyspace name '%s' doesn't match index name '%s'", keyspaceName, tableName);
+                throw ire(KEYSPACE_DOES_NOT_MATCH_INDEX, keyspaceName, tableName);
+            
+            // Set the configured default 2i implementation if one isn't specified with USING:
+            if (attrs.customClass == null)
+            {
+                if (DatabaseDescriptor.getDefaultSecondaryIndexEnabled())
+                    attrs.customClass = DatabaseDescriptor.getDefaultSecondaryIndex();
+                else
+                    // However, operators may require an implementation be specified
+                    throw ire(MUST_SPECIFY_INDEX_IMPLEMENTATION);
+            }
+            
+            // If we explicitly specify the index type "legacy_local_table", we can just clear the custom class, and the
+            // non-custom 2i creation process will begin. Otherwise, if an index type has been specified with 
+            // USING, make sure the appropriate custom index is created.
+            if (attrs.customClass != null)
+            {
+                if (!attrs.isCustom && attrs.customClass.equalsIgnoreCase(CassandraIndex.NAME))
+                    attrs.customClass = null;
+                else
+                    attrs.isCustom = true;
+            }
 
             return new CreateIndexStatement(keyspaceName, tableName.getName(), indexName.getName(), rawIndexTargets, attrs, ifNotExists);
         }
