@@ -38,6 +38,8 @@ import org.slf4j.LoggerFactory;
 import io.netty.util.concurrent.FastThreadLocal;
 import org.apache.cassandra.config.*;
 import org.apache.cassandra.db.filter.*;
+import org.apache.cassandra.db.transform.BasePartitions;
+import org.apache.cassandra.db.transform.BaseRows;
 import org.apache.cassandra.exceptions.QueryCancelledException;
 import org.apache.cassandra.net.MessageFlag;
 import org.apache.cassandra.net.MessagingService;
@@ -106,6 +108,8 @@ public abstract class ReadCommand extends AbstractReadQuery
 
     private boolean trackWarnings;
 
+    protected final DataRange dataRange;
+
     @Nullable
     private final Index.QueryPlan indexQueryPlan;
 
@@ -147,7 +151,8 @@ public abstract class ReadCommand extends AbstractReadQuery
                           RowFilter rowFilter,
                           DataLimits limits,
                           Index.QueryPlan indexQueryPlan,
-                          boolean trackWarnings)
+                          boolean trackWarnings,
+                          DataRange dataRange)
     {
         super(metadata, nowInSec, columnFilter, rowFilter, limits);
         if (acceptsTransient && isDigestQuery)
@@ -159,6 +164,7 @@ public abstract class ReadCommand extends AbstractReadQuery
         this.acceptsTransient = acceptsTransient;
         this.indexQueryPlan = indexQueryPlan;
         this.trackWarnings = trackWarnings;
+        this.dataRange = dataRange;
     }
 
     public static ReadCommand getCommand()
@@ -290,6 +296,12 @@ public abstract class ReadCommand extends AbstractReadQuery
      */
     public abstract ClusteringIndexFilter clusteringIndexFilter(DecoratedKey key);
 
+    @Override
+    public DataRange dataRange()
+    {
+        return dataRange;
+    }
+
     /**
      * Returns a copy of this command.
      *
@@ -394,6 +406,7 @@ public abstract class ReadCommand extends AbstractReadQuery
      * validation method to check that nothing in this command's parameters
      * violates the implementation specific validation rules.
      */
+    @Override
     public void maybeValidateIndex()
     {
         if (null != indexQueryPlan)
@@ -731,6 +744,22 @@ public abstract class ReadCommand extends AbstractReadQuery
     {
         long lastCheckedAt = 0;
 
+        @Override
+        protected void attachTo(BasePartitions partitions)
+        {
+            Preconditions.checkArgument(this.partitions == null || this.partitions == partitions,
+                                        "Attempted to attach 2nd different BasePartitions in StoppingTransformation; this is a bug.");
+            this.partitions = partitions;
+        }
+
+        @Override
+        protected void attachTo(BaseRows rows)
+        {
+            Preconditions.checkArgument(this.rows == null || this.rows == rows,
+                                        "Attempted to attach 2nd different BaseRows in StoppingTransformation; this is a bug.");
+            this.rows = rows;
+        }
+
         @Override
         protected UnfilteredRowIterator applyToPartition(UnfilteredRowIterator partition)
         {
