@@ -298,6 +298,7 @@ public abstract class ReadResponse
     {
         public void serialize(ReadResponse response, DataOutputPlus out, int version) throws IOException
         {
+            assert version >= MessagingService.VERSION_40;
             boolean isDigest = response instanceof DigestResponse;
             ByteBuffer digest = isDigest ? ((DigestResponse)response).digest : ByteBufferUtil.EMPTY_BYTE_BUFFER;
             ByteBufferUtil.writeWithVIntLength(digest, out);
@@ -312,12 +313,8 @@ public abstract class ReadResponse
                 // repaired sstables were read (but they might be on other replicas).
                 // If the coordinator did not request this info, the response contains an empty digest
                 // and a true for the isConclusive flag.
-                // If the messaging version is < 4.0, these are omitted altogether.
-                if (version >= MessagingService.VERSION_40)
-                {
-                    ByteBufferUtil.writeWithVIntLength(response.repairedDataDigest(), out);
-                    out.writeBoolean(response.isRepairedDigestConclusive());
-                }
+                ByteBufferUtil.writeWithVIntLength(response.repairedDataDigest(), out);
+                out.writeBoolean(response.isRepairedDigestConclusive());
 
                 ByteBuffer data = ((DataResponse)response).data;
                 ByteBufferUtil.writeWithVIntLength(data, out);
@@ -326,6 +323,7 @@ public abstract class ReadResponse
 
         public ReadResponse deserialize(DataInputPlus in, int version) throws IOException
         {
+            assert version >= MessagingService.VERSION_40;
             ByteBuffer digest = ByteBufferUtil.readWithVIntLength(in);
             if (digest.hasRemaining())
                 return new DigestResponse(digest);
@@ -334,17 +332,8 @@ public abstract class ReadResponse
             // that comes from the replica's repaired set, along with a flag indicating
             // whether or not the digest may be influenced by unrepaired/pending
             // repaired data
-            boolean repairedDigestConclusive;
-            if (version >= MessagingService.VERSION_40)
-            {
-                digest = ByteBufferUtil.readWithVIntLength(in);
-                repairedDigestConclusive = in.readBoolean();
-            }
-            else
-            {
-                digest = ByteBufferUtil.EMPTY_BYTE_BUFFER;
-                repairedDigestConclusive = true;
-            }
+            digest = ByteBufferUtil.readWithVIntLength(in);
+            boolean repairedDigestConclusive = in.readBoolean();
 
             ByteBuffer data = ByteBufferUtil.readWithVIntLength(in);
             return new RemoteDataResponse(data, digest, repairedDigestConclusive, version);
@@ -352,6 +341,7 @@ public abstract class ReadResponse
 
         public long serializedSize(ReadResponse response, int version)
         {
+            assert version >= MessagingService.VERSION_40;
             boolean isDigest = response instanceof DigestResponse;
             ByteBuffer digest = isDigest ? ((DigestResponse)response).digest : ByteBufferUtil.EMPTY_BYTE_BUFFER;
             long size = ByteBufferUtil.serializedSizeWithVIntLength(digest);
@@ -360,16 +350,12 @@ public abstract class ReadResponse
             {
                 // From 4.0, a coordinator may request an additional info about the repaired data
                 // that makes up the response.
-                if (version >= MessagingService.VERSION_40)
-                {
-                    size += ByteBufferUtil.serializedSizeWithVIntLength(response.repairedDataDigest());
-                    size += 1;
-                }
+                size += ByteBufferUtil.serializedSizeWithVIntLength(response.repairedDataDigest());
+                size += 1;
 
                 // In theory, we should deserialize/re-serialize if the version asked is different from the current
                 // version as the content could have a different serialization format. So far though, we haven't made
                 // change to partition iterators serialization since 3.0 so we skip this.
-                assert version >= MessagingService.VERSION_30;
                 ByteBuffer data = ((DataResponse)response).data;
                 size += ByteBufferUtil.serializedSizeWithVIntLength(data);
             }
