@@ -37,6 +37,8 @@ import com.google.common.collect.HashMultimap;
 import com.google.common.collect.Iterables;
 import com.google.common.collect.Multimap;
 import com.google.common.collect.Ordering;
+
+import org.apache.cassandra.io.util.File;
 import org.apache.commons.lang3.StringUtils;
 
 import org.apache.cassandra.utils.concurrent.Future;
@@ -45,7 +47,6 @@ import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import org.apache.cassandra.concurrent.Stage;
-import org.apache.cassandra.config.Config;
 import org.apache.cassandra.config.DatabaseDescriptor;
 import org.apache.cassandra.db.ColumnFamilyStore;
 import org.apache.cassandra.db.Keyspace;
@@ -53,7 +54,6 @@ import org.apache.cassandra.db.Mutation;
 import org.apache.cassandra.db.SystemKeyspace;
 import org.apache.cassandra.db.partitions.PartitionUpdate;
 import org.apache.cassandra.io.sstable.format.SSTableReader;
-import org.apache.cassandra.io.util.File;
 import org.apache.cassandra.io.util.FileUtils;
 import org.apache.cassandra.io.util.RandomAccessReader;
 import org.apache.cassandra.schema.Schema;
@@ -63,15 +63,20 @@ import org.apache.cassandra.schema.TableMetadataRef;
 import org.apache.cassandra.utils.FBUtilities;
 import org.apache.cassandra.utils.WrappedRunnable;
 
+import static java.lang.String.format;
+import static org.apache.cassandra.config.CassandraRelevantProperties.COMMITLOG_IGNORE_REPLAY_ERRORS;
+import static org.apache.cassandra.config.CassandraRelevantProperties.COMMITLOG_MAX_OUTSTANDING_REPLAY_BYTES;
+import static org.apache.cassandra.config.CassandraRelevantProperties.COMMITLOG_MAX_OUTSTANDING_REPLAY_COUNT;
+import static org.apache.cassandra.config.CassandraRelevantProperties.COMMIT_LOG_REPLAY_LIST;
+
 public class CommitLogReplayer implements CommitLogReadHandler
 {
     @VisibleForTesting
-    public static long MAX_OUTSTANDING_REPLAY_BYTES = Long.getLong("cassandra.commitlog_max_outstanding_replay_bytes", 1024 * 1024 * 64);
+    public static long MAX_OUTSTANDING_REPLAY_BYTES = COMMITLOG_MAX_OUTSTANDING_REPLAY_BYTES.getLong();
     @VisibleForTesting
     public static MutationInitiator mutationInitiator = new MutationInitiator();
-    static final String IGNORE_REPLAY_ERRORS_PROPERTY = Config.PROPERTY_PREFIX + "commitlog.ignorereplayerrors";
     private static final Logger logger = LoggerFactory.getLogger(CommitLogReplayer.class);
-    private static final int MAX_OUTSTANDING_REPLAY_COUNT = Integer.getInteger(Config.PROPERTY_PREFIX + "commitlog_max_outstanding_replay_count", 1024);
+    private static final int MAX_OUTSTANDING_REPLAY_COUNT = COMMITLOG_MAX_OUTSTANDING_REPLAY_COUNT.getInt();
 
     private final Set<Keyspace> keyspacesReplayed;
     private final Queue<Future<Integer>> futures;
@@ -382,28 +387,60 @@ public class CommitLogReplayer implements CommitLogReadHandler
 
         public abstract boolean includes(TableMetadataRef metadata);
 
+        /**
+         * Creates filter for entities to replay mutations for upon commit log replay.
+         *
+         * @see org.apache.cassandra.config.CassandraRelevantProperties#COMMIT_LOG_REPLAY_LIST
+         * */
         public static ReplayFilter create()
         {
-            // If no replaylist is supplied an empty array of strings is used to replay everything.
-            if (System.getProperty("cassandra.replayList") == null)
+            String replayList = COMMIT_LOG_REPLAY_LIST.getString();
+
+            if (replayList == null)
                 return new AlwaysReplayFilter();
 
             Multimap<String, String> toReplay = HashMultimap.create();
-            for (String rawPair : System.getProperty("cassandra.replayList").split(","))
+            for (String rawPair : replayList.split(","))
             {
-                String[] pair = StringUtils.split(rawPair.trim(), '.');
-                if (pair.length != 2)
-                    throw new IllegalArgumentException("Each table to be replayed must be fully qualified with keyspace name, e.g., 'system.peers'");
+                String trimmedRawPair = rawPair.trim();
+                if (trimmedRawPair.isEmpty() || trimmedRawPair.endsWith("."))
+                    throw new IllegalArgumentException(format("Invalid pair: '%s'", trimmedRawPair));
 
-                Keyspace ks = Schema.instance.getKeyspaceInstance(pair[0]);
+                String[] pair = StringUtils.split(trimmedRawPair, '.');
+
+                if (pair.length > 2)
+                    throw new IllegalArgumentException(format("%s property contains an item which " +
+                                                              "is not in format 'keyspace' or 'keyspace.table' " +
+                                                              "but it is '%s'",
+                                                              COMMIT_LOG_REPLAY_LIST.getKey(),
+                                                              String.join(".", pair)));
+
+                String keyspaceName = pair[0];
+
+                Keyspace ks = Schema.instance.getKeyspaceInstance(keyspaceName);
                 if (ks == null)
-                    throw new IllegalArgumentException("Unknown keyspace " + pair[0]);
-                ColumnFamilyStore cfs = ks.getColumnFamilyStore(pair[1]);
-                if (cfs == null)
-                    throw new IllegalArgumentException(String.format("Unknown table %s.%s", pair[0], pair[1]));
+                    throw new IllegalArgumentException("Unknown keyspace " + keyspaceName);
 
-                toReplay.put(pair[0], pair[1]);
+                if (pair.length == 1)
+                {
+                    for (ColumnFamilyStore cfs : ks.getColumnFamilyStores())
+                        toReplay.put(keyspaceName, cfs.name);
+                }
+                else
+                {
+                    ColumnFamilyStore cfs = ks.getColumnFamilyStore(pair[1]);
+                    if (cfs == null)
+                        throw new IllegalArgumentException(format("Unknown table %s.%s", keyspaceName, pair[1]));
+
+                    toReplay.put(keyspaceName, pair[1]);
+                }
             }
+
+            if (toReplay.isEmpty())
+                logger.info("All tables will be included in commit log replay.");
+            else
+                logger.info("Tables to be replayed: {}", toReplay.asMap().toString());
+
             return new CustomReplayFilter(toReplay);
         }
     }
@@ -497,13 +534,13 @@ public class CommitLogReplayer implements CommitLogReadHandler
     {
         if (exception.permissible)
             logger.error("Ignoring commit log replay error likely due to incomplete flush to disk", exception);
-        else if (Boolean.getBoolean(IGNORE_REPLAY_ERRORS_PROPERTY))
+        else if (COMMITLOG_IGNORE_REPLAY_ERRORS.getBoolean())
             logger.error("Ignoring commit log replay error", exception);
         else if (!CommitLog.handleCommitError("Failed commit log replay", exception))
         {
             logger.error("Replay stopped. If you wish to override this error and continue starting the node ignoring " +
-                         "commit log replay problems, specify -D" + IGNORE_REPLAY_ERRORS_PROPERTY + "=true " +
-                         "on the command line");
+                         "commit log replay problems, specify -D{}=true on the command line",
+                         COMMITLOG_IGNORE_REPLAY_ERRORS.getKey());
             throw new CommitLogReplayException(exception.getMessage(), exception);
         }
         return false;
