@@ -18,7 +18,6 @@
 package org.apache.cassandra.db.compaction;
 
 import java.time.Instant;
-import java.util.ArrayList;
 import java.util.Collection;
 import java.util.HashMap;
 import java.util.HashSet;
@@ -28,6 +27,7 @@ import java.util.Set;
 import java.util.concurrent.TimeUnit;
 
 import com.google.common.base.Predicate;
+import com.google.common.collect.ImmutableMap;
 import com.google.common.collect.Iterables;
 import com.google.common.collect.Sets;
 import com.google.common.util.concurrent.RateLimiter;
@@ -39,23 +39,18 @@ import org.apache.cassandra.config.DatabaseDescriptor;
 import org.apache.cassandra.db.ColumnFamilyStore;
 import org.apache.cassandra.db.Directories;
 import org.apache.cassandra.db.SystemKeyspace;
-import org.apache.cassandra.db.WriteContext;
 import org.apache.cassandra.db.compaction.writers.CompactionAwareWriter;
 import org.apache.cassandra.db.compaction.writers.DefaultCompactionWriter;
 import org.apache.cassandra.db.lifecycle.LifecycleTransaction;
-import org.apache.cassandra.db.rows.Row;
-import org.apache.cassandra.db.rows.Unfiltered;
-import org.apache.cassandra.db.rows.UnfilteredRowIterator;
-import org.apache.cassandra.index.Index;
-import org.apache.cassandra.index.transactions.IndexTransaction;
-import org.apache.cassandra.io.sstable.ISSTableScanner;
 import org.apache.cassandra.io.sstable.format.SSTableReader;
 import org.apache.cassandra.io.sstable.metadata.MetadataCollector;
+import org.apache.cassandra.io.util.File;
 import org.apache.cassandra.service.ActiveRepairService;
 import org.apache.cassandra.utils.FBUtilities;
 import org.apache.cassandra.utils.TimeUUID;
 import org.apache.cassandra.utils.concurrent.Refs;
 
+import static org.apache.cassandra.db.compaction.CompactionHistoryTabularData.COMPACTION_TYPE_PROPERTY;
 import static org.apache.cassandra.utils.Clock.Global.currentTimeMillis;
 import static org.apache.cassandra.utils.Clock.Global.nanoTime;
 import static org.apache.cassandra.utils.FBUtilities.now;
@@ -63,17 +58,17 @@ import static org.apache.cassandra.utils.FBUtilities.now;
 public class CompactionTask extends AbstractCompactionTask
 {
     protected static final Logger logger = LoggerFactory.getLogger(CompactionTask.class);
-    protected final int gcBefore;
+    protected final long gcBefore;
     protected final boolean keepOriginals;
     protected static long totalBytesCompacted = 0;
     private ActiveCompactionsTracker activeCompactions;
 
-    public CompactionTask(ColumnFamilyStore cfs, LifecycleTransaction txn, int gcBefore)
+    public CompactionTask(ColumnFamilyStore cfs, LifecycleTransaction txn, long gcBefore)
     {
         this(cfs, txn, gcBefore, false);
     }
 
-    public CompactionTask(ColumnFamilyStore cfs, LifecycleTransaction txn, int gcBefore, boolean keepOriginals)
+    public CompactionTask(ColumnFamilyStore cfs, LifecycleTransaction txn, long gcBefore, boolean keepOriginals)
     {
         super(cfs, txn);
         this.gcBefore = gcBefore;
@@ -140,8 +135,9 @@ public class CompactionTask extends AbstractCompactionTask
 
             final Set<SSTableReader> fullyExpiredSSTables = controller.getFullyExpiredSSTables();
 
+            TimeUUID taskId = transaction.opId();
             // select SSTables to compact based on available disk space.
-            if (!buildCompactionCandidatesForAvailableDiskSpace(fullyExpiredSSTables))
+            if (!buildCompactionCandidatesForAvailableDiskSpace(fullyExpiredSSTables, taskId))
             {
                 // The set of sstables has changed (one or more were excluded due to limited available disk space).
                 // We need to recompute the overlaps between sstables.
@@ -158,8 +154,6 @@ public class CompactionTask extends AbstractCompactionTask
                 }
             });
 
-            TimeUUID taskId = transaction.opId();
-
             // new sstables from flush can be added during a compaction, but only the compaction can remove them,
             // so in our single-threaded compaction world this is a valid way of determining if we're compacting
             // all the sstables (that existed when we started)
@@ -180,15 +174,13 @@ public class CompactionTask extends AbstractCompactionTask
             long inputSizeBytes;
             long timeSpentWritingKeys;
 
-            maybeNotifyIndexersAboutRowsInFullyExpiredSSTables(fullyExpiredSSTables);
-
             Set<SSTableReader> actuallyCompact = Sets.difference(transaction.originals(), fullyExpiredSSTables);
             Collection<SSTableReader> newSStables;
 
             long[] mergedRowCounts;
             long totalSourceCQLRows;
 
-            int nowInSec = FBUtilities.nowInSeconds();
+            long nowInSec = FBUtilities.nowInSeconds();
             try (Refs<SSTableReader> refs = Refs.ref(actuallyCompact);
                  AbstractCompactionStrategy.ScannerList scanners = strategy.getScanners(actuallyCompact);
                  CompactionIterator ci = new CompactionIterator(compactionType, scanners.scanners, controller, nowInSec, taskId))
@@ -216,6 +208,7 @@ public class CompactionTask extends AbstractCompactionTask
                         if (writer.append(ci.next()))
                             totalKeysWritten++;
 
+                        ci.setTargetDirectory(writer.getSStableDirectory().path());
                         long bytesScanned = scanners.getTotalBytesScanned();
 
                         // Rate limit the scanners, and account for compression
@@ -254,12 +247,13 @@ public class CompactionTask extends AbstractCompactionTask
 
             StringBuilder newSSTableNames = new StringBuilder();
             for (SSTableReader reader : newSStables)
-                newSSTableNames.append(reader.descriptor.baseFilename()).append(",");
+                newSSTableNames.append(reader.descriptor.baseFile()).append(",");
             long totalSourceRows = 0;
             for (int i = 0; i < mergedRowCounts.length; i++)
                 totalSourceRows += mergedRowCounts[i] * (i + 1);
 
-            String mergeSummary = updateCompactionHistory(taskId, cfs.keyspace.getName(), cfs.getTableName(), mergedRowCounts, startsize, endsize);
+            String mergeSummary = updateCompactionHistory(taskId, cfs.getKeyspaceName(), cfs.getTableName(), mergedRowCounts, startsize, endsize,
+                                                          ImmutableMap.of(COMPACTION_TYPE_PROPERTY, compactionType.type));
 
             logger.info(String.format("Compacted (%s) %d sstables to [%s] to level=%d.  %s to %s (~%d%% of original) in %,dms.  Read Throughput = %s, Write Throughput = %s, Row Throughput = ~%,d/s.  %,d total partitions merged to %,d.  Partition merge counts were {%s}. Time spent writing keys = %,dms",
                                        taskId,
@@ -298,7 +292,7 @@ public class CompactionTask extends AbstractCompactionTask
         return new DefaultCompactionWriter(cfs, directories, transaction, nonExpiredSSTables, keepOriginals, getLevel());
     }
 
-    public static String updateCompactionHistory(TimeUUID taskId, String keyspaceName, String columnFamilyName, long[] mergedRowCounts, long startSize, long endSize)
+    public static String updateCompactionHistory(TimeUUID taskId, String keyspaceName, String columnFamilyName, long[] mergedRowCounts, long startSize, long endSize, Map<String, String> compactionProperties)
     {
         StringBuilder mergeSummary = new StringBuilder(mergedRowCounts.length * 10);
         Map<Integer, Long> mergedRows = new HashMap<>();
@@ -312,7 +306,7 @@ public class CompactionTask extends AbstractCompactionTask
             mergeSummary.append(String.format("%d:%d, ", rows, count));
             mergedRows.put(rows, count);
         }
-        SystemKeyspace.updateCompactionHistory(taskId, keyspaceName, columnFamilyName, currentTimeMillis(), startSize, endSize, mergedRows);
+        SystemKeyspace.updateCompactionHistory(taskId, keyspaceName, columnFamilyName, currentTimeMillis(), startSize, endSize, mergedRows, compactionProperties);
         return mergeSummary.toString();
     }
 
@@ -367,12 +361,11 @@ public class CompactionTask extends AbstractCompactionTask
 
     /*
      * Checks if we have enough disk space to execute the compaction.  Drops the largest sstable out of the Task until
-     * there's enough space (in theory) to handle the compaction.  Does not take into account space that will be taken by
-     * other compactions.
+     * there's enough space (in theory) to handle the compaction.
      *
      * @return true if there is enough disk space to execute the complete compaction, false if some sstables are excluded.
      */
-    protected boolean buildCompactionCandidatesForAvailableDiskSpace(final Set<SSTableReader> fullyExpiredSSTables)
+    protected boolean buildCompactionCandidatesForAvailableDiskSpace(final Set<SSTableReader> fullyExpiredSSTables, TimeUUID taskId)
     {
         if(!cfs.isCompactionDiskSpaceCheckEnabled() && compactionType == OperationType.COMPACTION)
         {
@@ -387,13 +380,29 @@ public class CompactionTask extends AbstractCompactionTask
         while(!nonExpiredSSTables.isEmpty())
         {
             // Only consider write size of non expired SSTables
-            long expectedWriteSize = cfs.getExpectedCompactedFileSize(nonExpiredSSTables, compactionType);
-            long estimatedSSTables = Math.max(1, expectedWriteSize / strategy.getMaxSSTableBytes());
+            long writeSize;
+            try
+            {
+                writeSize = cfs.getExpectedCompactedFileSize(nonExpiredSSTables, compactionType);
+                Map<File, Long> expectedNewWriteSize = new HashMap<>();
+                List<File> newCompactionDatadirs = cfs.getDirectoriesForFiles(nonExpiredSSTables);
+                long writeSizePerOutputDatadir = writeSize / Math.max(newCompactionDatadirs.size(), 1);
+                for (File directory : newCompactionDatadirs)
+                    expectedNewWriteSize.put(directory, writeSizePerOutputDatadir);
 
-            if(cfs.getDirectories().hasAvailableDiskSpace(estimatedSSTables, expectedWriteSize))
+                Map<File, Long> expectedWriteSize = CompactionManager.instance.active.estimatedRemainingWriteBytes();
+
+                // todo: abort streams if they block compactions
+                if (cfs.getDirectories().hasDiskSpaceForCompactionsAndStreams(expectedNewWriteSize, expectedWriteSize))
+                    break;
+            }
+            catch (Exception e)
+            {
+                logger.error("Could not check if there is enough disk space for compaction {}", taskId, e);
                 break;
+            }
 
-            if (!reduceScopeForLimitedSpace(nonExpiredSSTables, expectedWriteSize))
+            if (!reduceScopeForLimitedSpace(nonExpiredSSTables, writeSize))
             {
                 // we end up here if we can't take any more sstables out of the compaction.
                 // usually means we've run out of disk space
@@ -406,15 +415,20 @@ public class CompactionTask extends AbstractCompactionTask
                     break;
                 }
 
-                String msg = String.format("Not enough space for compaction, estimated sstables = %d, expected write size = %d", estimatedSSTables, expectedWriteSize);
+                String msg = String.format("Not enough space for compaction (%s) of %s.%s, estimated sstables = %d, expected write size = %d",
+                                           taskId,
+                                           cfs.getKeyspaceName(),
+                                           cfs.name,
+                                           Math.max(1, writeSize / strategy.getMaxSSTableBytes()),
+                                           writeSize);
                 logger.warn(msg);
                 CompactionManager.instance.incrementAborted();
                 throw new RuntimeException(msg);
             }
 
             sstablesRemoved++;
-            logger.warn("Not enough space for compaction, {}MiB estimated.  Reducing scope.",
-                        (float) expectedWriteSize / 1024 / 1024);
+            logger.warn("Not enough space for compaction {}, {}MiB estimated. Reducing scope.",
+                        taskId, (float) writeSize / 1024 / 1024);
         }
 
         if(sstablesRemoved > 0)
@@ -451,65 +465,4 @@ public class CompactionTask extends AbstractCompactionTask
         }
         return max;
     }
-
-    private void maybeNotifyIndexersAboutRowsInFullyExpiredSSTables(Set<SSTableReader> fullyExpiredSSTables)
-    {
-        if (fullyExpiredSSTables.isEmpty())
-            return;
-
-        List<Index> indexes = new ArrayList<>();
-        for (Index index : cfs.indexManager.listIndexes())
-        {
-            if (index.notifyIndexerAboutRowsInFullyExpiredSSTables())
-                indexes.add(index);
-        }
-
-        if (indexes.isEmpty())
-            return;
-
-        for (SSTableReader expiredSSTable : fullyExpiredSSTables)
-        {
-            try (ISSTableScanner scanner = expiredSSTable.getScanner())
-            {
-                while (scanner.hasNext())
-                {
-                    try (UnfilteredRowIterator partition = scanner.next();
-                         WriteContext ctx = cfs.keyspace.getWriteHandler().createContextForIndexing())
-                    {
-                        List<Index.Indexer> indexers = new ArrayList<>();
-                        for (int i = 0; i < indexes.size(); i++)
-                        {
-                            Index.Indexer indexer = indexes.get(i).indexerFor(partition.partitionKey(),
-                                                                              partition.columns(),
-                                                                              FBUtilities.nowInSeconds(),
-                                                                              ctx,
-                                                                              IndexTransaction.Type.COMPACTION);
-
-                            if (indexer != null)
-                                indexers.add(indexer);
-                        }
-
-                        if (!indexers.isEmpty())
-                        {
-                            for (Index.Indexer indexer : indexers)
-                                indexer.begin();
-
-                            while (partition.hasNext())
-                            {
-                                Unfiltered unfiltered = partition.next();
-                                if (unfiltered instanceof Row)
-                                {
-                                    for (Index.Indexer indexer : indexers)
-                                        indexer.removeRow((Row) unfiltered);
-                                }
-                            }
-
-                            for (Index.Indexer indexer : indexers)
-                                indexer.finish();
-                        }
-                    }
-                }
-            }
-        }
-    }
 }
