@@ -18,18 +18,15 @@
 package org.apache.cassandra.db.filter;
 
 import java.io.IOException;
-import java.util.List;
-import java.nio.ByteBuffer;
 
-import org.apache.cassandra.schema.TableMetadata;
 import org.apache.cassandra.db.*;
-import org.apache.cassandra.db.rows.*;
 import org.apache.cassandra.db.partitions.CachedPartition;
 import org.apache.cassandra.db.partitions.Partition;
+import org.apache.cassandra.db.rows.*;
 import org.apache.cassandra.db.transform.Transformation;
-import org.apache.cassandra.io.sstable.format.SSTableReader;
 import org.apache.cassandra.io.util.DataInputPlus;
 import org.apache.cassandra.io.util.DataOutputPlus;
+import org.apache.cassandra.schema.TableMetadata;
 
 /**
  * A filter over a single partition.
@@ -126,15 +123,9 @@ public class ClusteringIndexSliceFilter extends AbstractClusteringIndexFilter
         return partition.unfilteredIterator(columnFilter, slices, reversed);
     }
 
-    public boolean shouldInclude(SSTableReader sstable)
+    public boolean intersects(ClusteringComparator comparator, Slice slice)
     {
-        List<ByteBuffer> minClusteringValues = sstable.getSSTableMetadata().minClusteringValues;
-        List<ByteBuffer> maxClusteringValues = sstable.getSSTableMetadata().maxClusteringValues;
-
-        if (minClusteringValues.isEmpty() || maxClusteringValues.isEmpty())
-            return true;
-
-        return slices.intersects(minClusteringValues, maxClusteringValues);
+        return slices.intersects(slice);
     }
 
     public String toString(TableMetadata metadata)
@@ -176,4 +167,4 @@ public class ClusteringIndexSliceFilter extends AbstractClusteringIndexFilter
             return new ClusteringIndexSliceFilter(slices, reversed);
         }
     }
-}
+}
\ No newline at end of file
