@@ -400,6 +400,19 @@ public final class Guardrails implements GuardrailsMBean
                                : format("User types cannot have more than %s columns, but %s provided for user type %s.",
                                         threshold, value, what));
 
+    /**
+     * Guardrail on the usage of vector type.
+     *
+     * This may be useful when running clusters that have diverse clients, where some do not support the vector type.
+     * Clients that do not support vector may fail to establish sessions or fail to read results containing vector
+     * columns.
+     */
+    public static final EnableFlag vectorTypeEnabled =
+    new EnableFlag("vector_type_enabled",
+                     null,
+                     state -> CONFIG_PROVIDER.getOrCreate(state).getVectorTypeEnabled(),
+                    "usage of the vector type");
+
     /**
      * Guardrail on the number of dimensions of vector columns.
      */
@@ -1139,6 +1152,18 @@ public final class Guardrails implements GuardrailsMBean
         DEFAULT_CONFIG.setVectorDimensionsThreshold(warn, fail);
     }
 
+    @Override
+    public void setVectorTypeEnabled(boolean enabled)
+    {
+        DEFAULT_CONFIG.setVectorTypeEnabled(enabled);
+    }
+
+    @Override
+    public boolean getVectorTypeEnabled()
+    {
+        return DEFAULT_CONFIG.getVectorTypeEnabled();
+    }
+
     @Override
     public int getMaximumReplicationFactorWarnThreshold()
     {
