@@ -47,6 +47,8 @@ public class CassandraIncomingFile implements IncomingStream
     private volatile long size = -1;
     private volatile int numFiles = 1;
 
+    private volatile boolean isEntireSSTable = false;
+
     private static final Logger logger = LoggerFactory.getLogger(CassandraIncomingFile.class);
 
     public CassandraIncomingFile(ColumnFamilyStore cfs, StreamSession session, StreamMessageHeader header)
@@ -72,6 +74,7 @@ public class CassandraIncomingFile implements IncomingStream
         IStreamReader reader;
         if (streamHeader.isEntireSSTable)
         {
+            isEntireSSTable = true;
             reader = new CassandraEntireSSTableStreamReader(header, streamHeader, session);
             numFiles = streamHeader.componentManifest.components().size();
         }
@@ -103,6 +106,11 @@ public class CassandraIncomingFile implements IncomingStream
         return numFiles;
     }
 
+    public boolean isEntireSSTable()
+    {
+        return isEntireSSTable;
+    }
+    
     @Override
     public TableId getTableId()
     {
