@@ -23,20 +23,28 @@ import org.apache.cassandra.net.Message;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-public class GossipShutdownVerbHandler implements IVerbHandler
+public class GossipShutdownVerbHandler implements IVerbHandler<GossipShutdown>
 {
     public static final GossipShutdownVerbHandler instance = new GossipShutdownVerbHandler();
 
     private static final Logger logger = LoggerFactory.getLogger(GossipShutdownVerbHandler.class);
 
-    public void doVerb(Message message)
+    public void doVerb(Message<GossipShutdown> message)
     {
         if (!Gossiper.instance.isEnabled())
         {
             logger.debug("Ignoring shutdown message from {} because gossip is disabled", message.from());
             return;
         }
-        Gossiper.instance.markAsShutdown(message.from());
+        HeartBeatState previous = Gossiper.instance.getEndpointStateForEndpoint(message.from()).getHeartBeatState();
+
+        if (message.payload == null)
+            Gossiper.instance.markAsShutdown(message.from());
+        else if (previous.getGeneration() <= message.payload.state.getHeartBeatState().getGeneration())
+            Gossiper.instance.markAsShutdown(message.from(), message.payload.state);
+        else
+            logger.debug("Ignoring shutdown message from {} because generation {} older than local {}",
+                    message.from(), message.payload.state.getHeartBeatState().getGeneration(), previous.getGeneration());
     }
 
 }
