@@ -18,21 +18,25 @@
 package org.apache.cassandra.io.sstable;
 
 
-import java.io.IOException;
 import java.io.Closeable;
+import java.io.IOException;
 import java.nio.ByteBuffer;
 import java.nio.file.Files;
 import java.nio.file.Path;
+import java.util.ArrayList;
 import java.util.Collection;
-import java.util.Collections;
 import java.util.Objects;
 import java.util.concurrent.atomic.AtomicReference;
 import java.util.function.Consumer;
 import java.util.stream.Stream;
 
-import org.apache.cassandra.db.*;
+import org.apache.cassandra.config.DatabaseDescriptor;
+import org.apache.cassandra.db.DecoratedKey;
+import org.apache.cassandra.db.RegularAndStaticColumns;
+import org.apache.cassandra.db.SerializationHeader;
 import org.apache.cassandra.db.partitions.PartitionUpdate;
 import org.apache.cassandra.db.rows.EncodingStats;
+import org.apache.cassandra.index.Index;
 import org.apache.cassandra.io.sstable.format.SSTableFormat;
 import org.apache.cassandra.io.sstable.format.SSTableReader;
 import org.apache.cassandra.io.util.File;
@@ -47,9 +51,10 @@ abstract class AbstractSSTableSimpleWriter implements Closeable
     protected final File directory;
     protected final TableMetadataRef metadata;
     protected final RegularAndStaticColumns columns;
-    protected SSTableFormat.Type formatType = SSTableFormat.Type.current();
+    protected SSTableFormat<?, ?> format = DatabaseDescriptor.getSelectedSSTableFormat();
     protected static final AtomicReference<SSTableId> id = new AtomicReference<>(SSTableIdFactory.instance.defaultBuilder().generator(Stream.empty()).get());
     protected boolean makeRangeAware = false;
+    protected final Collection<Index.Group> indexGroups;
     protected Consumer<Collection<SSTableReader>> sstableProducedListener;
     protected boolean openSSTableOnProduced = false;
 
@@ -58,11 +63,12 @@ abstract class AbstractSSTableSimpleWriter implements Closeable
         this.metadata = metadata;
         this.directory = directory;
         this.columns = columns;
+        indexGroups = new ArrayList<>();
     }
 
-    protected void setSSTableFormatType(SSTableFormat.Type type)
+    protected void setSSTableFormatType(SSTableFormat<?, ?> type)
     {
-        this.formatType = type;
+        this.format = type;
     }
 
     protected void setRangeAwareWriting(boolean makeRangeAware)
@@ -70,6 +76,11 @@ abstract class AbstractSSTableSimpleWriter implements Closeable
         this.makeRangeAware = makeRangeAware;
     }
 
+    protected void addIndexGroup(Index.Group indexGroup)
+    {
+        this.indexGroups.add(indexGroup);
+    }
+
     protected void setSSTableProducedListener(Consumer<Collection<SSTableReader>> listener)
     {
         this.sstableProducedListener = Objects.requireNonNull(listener);
@@ -96,25 +107,25 @@ abstract class AbstractSSTableSimpleWriter implements Closeable
         sstableProducedListener.accept(sstables);
     }
 
-    protected SSTableTxnWriter createWriter() throws IOException
+    protected SSTableTxnWriter createWriter(SSTable.Owner owner) throws IOException
     {
         SerializationHeader header = new SerializationHeader(true, metadata.get(), columns, EncodingStats.NO_STATS);
 
         if (makeRangeAware)
-            return SSTableTxnWriter.createRangeAware(metadata, 0,  ActiveRepairService.UNREPAIRED_SSTABLE, ActiveRepairService.NO_PENDING_REPAIR, false, formatType, 0, header);
+            return SSTableTxnWriter.createRangeAware(metadata, 0, ActiveRepairService.UNREPAIRED_SSTABLE, ActiveRepairService.NO_PENDING_REPAIR, false, format, header);
 
         return SSTableTxnWriter.create(metadata,
-                                       createDescriptor(directory, metadata.keyspace, metadata.name, formatType),
+                                       createDescriptor(directory, metadata.keyspace, metadata.name, format),
                                        0,
                                        ActiveRepairService.UNREPAIRED_SSTABLE,
                                        ActiveRepairService.NO_PENDING_REPAIR,
                                        false,
-                                       0,
                                        header,
-                                       Collections.emptySet());
+                                       indexGroups,
+                                       owner);
     }
 
-    private static Descriptor createDescriptor(File directory, final String keyspace, final String columnFamily, final SSTableFormat.Type fmt) throws IOException
+    private static Descriptor createDescriptor(File directory, final String keyspace, final String columnFamily, final SSTableFormat<?, ?> fmt) throws IOException
     {
         SSTableId nextGen = getNextId(directory, columnFamily);
         return new Descriptor(directory, keyspace, columnFamily, nextGen, fmt);
@@ -127,7 +138,7 @@ abstract class AbstractSSTableSimpleWriter implements Closeable
             try (Stream<Path> existingPaths = Files.list(directory.toPath()))
             {
                 Stream<SSTableId> existingIds = existingPaths.map(File::new)
-                                                             .map(SSTable::tryDescriptorFromFilename)
+                                                             .map(SSTable::tryDescriptorFromFile)
                                                              .filter(d -> d != null && d.cfname.equals(columnFamily))
                                                              .map(d -> d.id);
 
