@@ -23,51 +23,30 @@ import java.nio.ByteBuffer;
 import java.util.zip.CheckedInputStream;
 import java.util.zip.Checksum;
 
-import com.google.common.annotations.VisibleForTesting;
-
-import org.apache.cassandra.io.sstable.Component;
-import org.apache.cassandra.io.sstable.Descriptor;
 import org.apache.cassandra.utils.ChecksumType;
-import org.apache.cassandra.utils.Throwables;
 
 public class DataIntegrityMetadata
 {
-    public static ChecksumValidator checksumValidator(Descriptor desc) throws IOException
-    {
-        return new ChecksumValidator(desc);
-    }
-
     public static class ChecksumValidator implements Closeable
     {
         private final ChecksumType checksumType;
         private final RandomAccessReader reader;
         public final int chunkSize;
-        private final String dataFilename;
 
-        public ChecksumValidator(Descriptor descriptor) throws IOException
+        public ChecksumValidator(File dataFile, File crcFile) throws IOException
         {
             this(ChecksumType.CRC32,
-                 RandomAccessReader.open(new File(descriptor.filenameFor(Component.CRC))),
-                 descriptor.filenameFor(Component.DATA));
+                 RandomAccessReader.open(crcFile),
+                 dataFile.absolutePath());
         }
 
         public ChecksumValidator(ChecksumType checksumType, RandomAccessReader reader, String dataFilename) throws IOException
         {
             this.checksumType = checksumType;
             this.reader = reader;
-            this.dataFilename = dataFilename;
             chunkSize = reader.readInt();
         }
 
-        @VisibleForTesting
-        protected ChecksumValidator(ChecksumType checksumType, RandomAccessReader reader, int chunkSize)
-        {
-            this.checksumType = checksumType;
-            this.reader = reader;
-            this.dataFilename = null;
-            this.chunkSize = chunkSize;
-        }
-
         public void seek(long offset)
         {
             long start = chunkStart(offset);
@@ -82,10 +61,10 @@ public class DataIntegrityMetadata
 
         public void validate(byte[] bytes, int start, int end) throws IOException
         {
-            int current = (int) checksumType.of(bytes, start, end);
-            int actual = reader.readInt();
-            if (current != actual)
-                throw new IOException("Corrupted File : " + dataFilename);
+            int calculatedValue = (int) checksumType.of(bytes, start, end);
+            int storedValue = reader.readInt();
+            if (calculatedValue != storedValue)
+                throw new IOException(String.format("Corrupted file: integrity check (%s) failed for %s: %d != %d", checksumType.name(), reader.getPath(), storedValue, calculatedValue));
         }
 
         /**
@@ -96,10 +75,10 @@ public class DataIntegrityMetadata
          */
         public void validate(ByteBuffer buffer) throws IOException
         {
-            int current = (int) checksumType.of(buffer);
-            int actual = reader.readInt();
-            if (current != actual)
-                throw new IOException("Corrupted File : " + dataFilename);
+            int calculatedValue = (int) checksumType.of(buffer);
+            int storedValue = reader.readInt();
+            if (calculatedValue != storedValue)
+                throw new IOException(String.format("Corrupted file: integrity check (%s) failed for %s: %d != %d", checksumType.name(), reader.getPath(), storedValue, calculatedValue));
         }
 
         public void close()
@@ -108,55 +87,33 @@ public class DataIntegrityMetadata
         }
     }
 
-    public static FileDigestValidator fileDigestValidator(Descriptor desc) throws IOException
-    {
-        return new FileDigestValidator(desc);
-    }
-
-    public static class FileDigestValidator implements Closeable
+    public static class FileDigestValidator
     {
         private final Checksum checksum;
-        private final RandomAccessReader digestReader;
-        private final RandomAccessReader dataReader;
-        private final Descriptor descriptor;
-        private long storedDigestValue;
+        private final File dataFile;
+        private final File digestFile;
 
-        public FileDigestValidator(Descriptor descriptor) throws IOException
+        public FileDigestValidator(File dataFile, File digestFile) throws IOException
         {
-            this.descriptor = descriptor;
-            checksum = ChecksumType.CRC32.newInstance();
-            digestReader = RandomAccessReader.open(new File(descriptor.filenameFor(Component.DIGEST)));
-            dataReader = RandomAccessReader.open(new File(descriptor.filenameFor(Component.DATA)));
-            try
-            {
-                storedDigestValue = Long.parseLong(digestReader.readLine());
-            }
-            catch (Exception e)
-            {
-                close();
-                // Attempting to create a FileDigestValidator without a DIGEST file will fail
-                throw new IOException("Corrupted SSTable : " + descriptor.filenameFor(Component.DATA));
-            }
+            this.dataFile = dataFile;
+            this.digestFile = digestFile;
+            this.checksum = ChecksumType.CRC32.newInstance();
         }
 
         // Validate the entire file
         public void validate() throws IOException
         {
-            CheckedInputStream checkedInputStream = new CheckedInputStream(dataReader, checksum);
-            byte[] chunk = new byte[64 * 1024];
-
-            while( checkedInputStream.read(chunk) > 0 ) { }
-            long calculatedDigestValue = checkedInputStream.getChecksum().getValue();
-            if (storedDigestValue != calculatedDigestValue)
+            try (RandomAccessReader digestReader = RandomAccessReader.open(digestFile);
+                 RandomAccessReader dataReader = RandomAccessReader.open(dataFile);
+                 CheckedInputStream checkedInputStream = new CheckedInputStream(dataReader, checksum);)
             {
-                throw new IOException("Corrupted SSTable : " + descriptor.filenameFor(Component.DATA));
+                long storedDigestValue = Long.parseLong(digestReader.readLine());
+                byte[] chunk = new byte[64 * 1024];
+                while (checkedInputStream.read(chunk) > 0) ;
+                long calculatedDigestValue = checkedInputStream.getChecksum().getValue();
+                if (storedDigestValue != calculatedDigestValue)
+                    throw new IOException(String.format("Corrupted file: integrity check (digest) failed for %s: %d != %d", dataFile, storedDigestValue, calculatedDigestValue));
             }
         }
-
-        public void close()
-        {
-            Throwables.perform(digestReader::close,
-                               dataReader::close);
-        }
     }
 }
