@@ -19,16 +19,17 @@ package org.apache.cassandra.repair;
 
 import java.util.*;
 import java.util.concurrent.CopyOnWriteArrayList;
-import java.util.function.Predicate;
+import java.util.concurrent.Executor;
 import java.util.function.Function;
+import java.util.function.Predicate;
 import java.util.stream.Collectors;
-
+import javax.annotation.Nullable;
 
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Preconditions;
 import com.google.common.collect.ImmutableMap;
 import com.google.common.util.concurrent.*;
-import org.apache.cassandra.concurrent.ExecutorPlus;
+
 import org.apache.cassandra.schema.Schema;
 import org.apache.cassandra.schema.TableMetadata;
 import org.apache.cassandra.repair.state.JobState;
@@ -39,17 +40,16 @@ import org.slf4j.LoggerFactory;
 import org.apache.cassandra.config.DatabaseDescriptor;
 import org.apache.cassandra.db.ColumnFamilyStore;
 import org.apache.cassandra.db.Keyspace;
+import org.apache.cassandra.dht.Range;
+import org.apache.cassandra.dht.Token;
+import org.apache.cassandra.locator.InetAddressAndPort;
 import org.apache.cassandra.repair.asymmetric.DifferenceHolder;
 import org.apache.cassandra.repair.asymmetric.HostDifferences;
 import org.apache.cassandra.repair.asymmetric.PreferedNodeFilter;
 import org.apache.cassandra.repair.asymmetric.ReduceHelper;
-import org.apache.cassandra.locator.InetAddressAndPort;
 import org.apache.cassandra.schema.SystemDistributedKeyspace;
-import org.apache.cassandra.service.ActiveRepairService;
-import org.apache.cassandra.streaming.PreviewKind;
-import org.apache.cassandra.dht.Range;
-import org.apache.cassandra.dht.Token;
 import org.apache.cassandra.service.paxos.cleanup.PaxosCleanup;
+import org.apache.cassandra.streaming.PreviewKind;
 import org.apache.cassandra.tracing.Tracing;
 import org.apache.cassandra.utils.FBUtilities;
 import org.apache.cassandra.utils.MerkleTrees;
@@ -60,7 +60,6 @@ import org.apache.cassandra.utils.concurrent.ImmediateFuture;
 
 import static org.apache.cassandra.config.DatabaseDescriptor.paxosRepairEnabled;
 import static org.apache.cassandra.service.paxos.Paxos.useV2;
-import static org.apache.cassandra.utils.Clock.Global.currentTimeMillis;
 
 /**
  * RepairJob runs repair on given ColumnFamily.
@@ -69,11 +68,12 @@ public class RepairJob extends AsyncFuture<RepairResult> implements Runnable
 {
     private static final Logger logger = LoggerFactory.getLogger(RepairJob.class);
 
+    private final SharedContext ctx;
     public final JobState state;
     private final RepairJobDesc desc;
     private final RepairSession session;
     private final RepairParallelism parallelismDegree;
-    private final ExecutorPlus taskExecutor;
+    private final Executor taskExecutor;
 
     @VisibleForTesting
     final List<ValidationTask> validationTasks = new CopyOnWriteArrayList<>();
@@ -88,16 +88,17 @@ public class RepairJob extends AsyncFuture<RepairResult> implements Runnable
      */
     public RepairJob(RepairSession session, String columnFamily)
     {
+        this.ctx = session.ctx;
         this.session = session;
         this.taskExecutor = session.taskExecutor;
         this.parallelismDegree = session.parallelismDegree;
         this.desc = new RepairJobDesc(session.state.parentRepairSession, session.getId(), session.state.keyspace, columnFamily, session.state.commonRange.ranges);
-        this.state = new JobState(desc, session.state.commonRange.endpoints);
+        this.state = new JobState(ctx.clock(), desc, session.state.commonRange.endpoints);
     }
 
-    public int getNowInSeconds()
+    public long getNowInSeconds()
     {
-        int nowInSeconds = FBUtilities.nowInSeconds();
+        long nowInSeconds = ctx.clock().nowInSeconds();
         if (session.previewKind == PreviewKind.REPAIRED)
         {
             return nowInSeconds + DatabaseDescriptor.getValidationPreviewPurgeHeadStartInSec();
@@ -121,14 +122,14 @@ public class RepairJob extends AsyncFuture<RepairResult> implements Runnable
         ColumnFamilyStore cfs = ks.getColumnFamilyStore(desc.columnFamily);
         cfs.metric.repairsStarted.inc();
         List<InetAddressAndPort> allEndpoints = new ArrayList<>(session.state.commonRange.endpoints);
-        allEndpoints.add(FBUtilities.getBroadcastAddressAndPort());
+        allEndpoints.add(ctx.broadcastAddressAndPort());
 
         Future<Void> paxosRepair;
         if (paxosRepairEnabled() && ((useV2() && session.repairPaxos) || session.paxosOnly))
         {
             logger.info("{} {}.{} starting paxos repair", session.previewKind.logPrefix(session.getId()), desc.keyspace, desc.columnFamily);
             TableMetadata metadata = Schema.instance.getTableMetadata(desc.keyspace, desc.columnFamily);
-            paxosRepair = PaxosCleanup.cleanup(allEndpoints, metadata, desc.ranges, session.state.commonRange.hasSkippedReplicas, taskExecutor);
+            paxosRepair = PaxosCleanup.cleanup(ctx, allEndpoints, metadata, desc.ranges, session.state.commonRange.hasSkippedReplicas, taskExecutor);
         }
         else
         {
@@ -138,7 +139,7 @@ public class RepairJob extends AsyncFuture<RepairResult> implements Runnable
 
         if (session.paxosOnly)
         {
-            paxosRepair.addCallback(new FutureCallback<Void>()
+            paxosRepair.addCallback(new FutureCallback<>()
             {
                 public void onSuccess(Void v)
                 {
@@ -175,7 +176,7 @@ public class RepairJob extends AsyncFuture<RepairResult> implements Runnable
                     state.phase.snapshotsSubmitted();
                     for (InetAddressAndPort endpoint : allEndpoints)
                     {
-                        SnapshotTask snapshotTask = new SnapshotTask(desc, endpoint);
+                        SnapshotTask snapshotTask = new SnapshotTask(ctx, desc, endpoint);
                         snapshotTasks.add(snapshotTask);
                         taskExecutor.execute(snapshotTask);
                     }
@@ -197,7 +198,7 @@ public class RepairJob extends AsyncFuture<RepairResult> implements Runnable
                                                                         .flatMap(this::executeTasks, taskExecutor);
 
         // When all sync complete, set the final result
-        syncResults.addCallback(new FutureCallback<List<SyncStat>>()
+        syncResults.addCallback(new FutureCallback<>()
         {
             @Override
             public void onSuccess(List<SyncStat> stats)
@@ -219,9 +220,7 @@ public class RepairJob extends AsyncFuture<RepairResult> implements Runnable
             public void onFailure(Throwable t)
             {
                 state.phase.fail(t);
-                // Make sure all validation tasks have cleaned up the off-heap Merkle trees they might contain.
-                validationTasks.forEach(ValidationTask::abort);
-                syncTasks.forEach(SyncTask::abort);
+                abort(t);
 
                 if (!session.previewKind.isPreview())
                 {
@@ -265,6 +264,17 @@ public class RepairJob extends AsyncFuture<RepairResult> implements Runnable
                                  : this::createStandardSyncTasks, taskExecutor);
     }
 
+    public synchronized void abort(@Nullable Throwable reason)
+    {
+        if (reason == null)
+            reason = new RuntimeException("Abort");
+        // Make sure all validation tasks have cleaned up the off-heap Merkle trees they might contain.
+        for (ValidationTask v : validationTasks)
+            v.abort(reason);
+        for (SyncTask s : syncTasks)
+            s.abort(reason);
+    }
+
     private boolean isTransient(InetAddressAndPort ep)
     {
         return session.state.commonRange.transEndpoints.contains(ep);
@@ -272,9 +282,9 @@ public class RepairJob extends AsyncFuture<RepairResult> implements Runnable
 
     private List<SyncTask> createStandardSyncTasks(List<TreeResponse> trees)
     {
-        return createStandardSyncTasks(desc,
+        return createStandardSyncTasks(ctx, desc,
                                        trees,
-                                       FBUtilities.getLocalAddressAndPort(),
+                                       ctx.broadcastAddressAndPort(),
                                        this::isTransient,
                                        session.isIncremental,
                                        session.pullRepair,
@@ -282,7 +292,8 @@ public class RepairJob extends AsyncFuture<RepairResult> implements Runnable
     }
 
     @VisibleForTesting
-    static List<SyncTask> createStandardSyncTasks(RepairJobDesc desc,
+    static List<SyncTask> createStandardSyncTasks(SharedContext ctx,
+                                                  RepairJobDesc desc,
                                                   List<TreeResponse> trees,
                                                   InetAddressAndPort local,
                                                   Predicate<InetAddressAndPort> isTransient,
@@ -290,7 +301,7 @@ public class RepairJob extends AsyncFuture<RepairResult> implements Runnable
                                                   boolean pullRepair,
                                                   PreviewKind previewKind)
     {
-        long startedAt = currentTimeMillis();
+        long startedAt = ctx.clock().currentTimeMillis();
         List<SyncTask> syncTasks = new ArrayList<>();
         // We need to difference all trees one against another
         for (int i = 0; i < trees.size() - 1; ++i)
@@ -325,7 +336,7 @@ public class RepairJob extends AsyncFuture<RepairResult> implements Runnable
                     if (!requestRanges && !transferRanges)
                         continue;
 
-                    task = new LocalSyncTask(desc, self.endpoint, remote.endpoint, differences, isIncremental ? desc.parentSessionId : null,
+                    task = new LocalSyncTask(ctx, desc, self.endpoint, remote.endpoint, differences, isIncremental ? desc.parentSessionId : null,
                                              requestRanges, transferRanges, previewKind);
                 }
                 else if (isTransient.test(r1.endpoint) || isTransient.test(r2.endpoint))
@@ -333,11 +344,11 @@ public class RepairJob extends AsyncFuture<RepairResult> implements Runnable
                     // Stream only from transient replica
                     TreeResponse streamFrom = isTransient.test(r1.endpoint) ? r1 : r2;
                     TreeResponse streamTo = isTransient.test(r1.endpoint) ? r2 : r1;
-                    task = new AsymmetricRemoteSyncTask(desc, streamTo.endpoint, streamFrom.endpoint, differences, previewKind);
+                    task = new AsymmetricRemoteSyncTask(ctx, desc, streamTo.endpoint, streamFrom.endpoint, differences, previewKind);
                 }
                 else
                 {
-                    task = new SymmetricRemoteSyncTask(desc, r1.endpoint, r2.endpoint, differences, previewKind);
+                    task = new SymmetricRemoteSyncTask(ctx, desc, r1.endpoint, r2.endpoint, differences, previewKind);
                 }
                 syncTasks.add(task);
             }
@@ -345,7 +356,7 @@ public class RepairJob extends AsyncFuture<RepairResult> implements Runnable
         }
         trees.get(trees.size() - 1).trees.release();
         logger.info("Created {} sync tasks based on {} merkle tree responses for {} (took: {}ms)",
-                    syncTasks.size(), trees.size(), desc.parentSessionId, currentTimeMillis() - startedAt);
+                    syncTasks.size(), trees.size(), desc.parentSessionId, ctx.clock().currentTimeMillis() - startedAt);
         return syncTasks;
     }
 
@@ -354,9 +365,12 @@ public class RepairJob extends AsyncFuture<RepairResult> implements Runnable
     {
         try
         {
-            ActiveRepairService.instance.getParentRepairSession(desc.parentSessionId);
+            ctx.repair().getParentRepairSession(desc.parentSessionId);
             syncTasks.addAll(tasks);
 
+            if (!tasks.isEmpty())
+                state.phase.streamSubmitted();
+
             for (SyncTask task : tasks)
             {
                 if (!task.isLocal())
@@ -389,7 +403,8 @@ public class RepairJob extends AsyncFuture<RepairResult> implements Runnable
 
     private List<SyncTask> createOptimisedSyncingSyncTasks(List<TreeResponse> trees)
     {
-        return createOptimisedSyncingSyncTasks(desc,
+        return createOptimisedSyncingSyncTasks(ctx,
+                                               desc,
                                                trees,
                                                FBUtilities.getLocalAddressAndPort(),
                                                this::isTransient,
@@ -398,7 +413,9 @@ public class RepairJob extends AsyncFuture<RepairResult> implements Runnable
                                                session.previewKind);
     }
 
-    static List<SyncTask> createOptimisedSyncingSyncTasks(RepairJobDesc desc,
+    @VisibleForTesting
+    static List<SyncTask> createOptimisedSyncingSyncTasks(SharedContext ctx,
+                                                          RepairJobDesc desc,
                                                           List<TreeResponse> trees,
                                                           InetAddressAndPort local,
                                                           Predicate<InetAddressAndPort> isTransient,
@@ -406,7 +423,7 @@ public class RepairJob extends AsyncFuture<RepairResult> implements Runnable
                                                           boolean isIncremental,
                                                           PreviewKind previewKind)
     {
-        long startedAt = currentTimeMillis();
+        long startedAt = ctx.clock().currentTimeMillis();
         List<SyncTask> syncTasks = new ArrayList<>();
         // We need to difference all trees one against another
         DifferenceHolder diffHolder = new DifferenceHolder(trees);
@@ -440,12 +457,12 @@ public class RepairJob extends AsyncFuture<RepairResult> implements Runnable
                     SyncTask task;
                     if (address.equals(local))
                     {
-                        task = new LocalSyncTask(desc, address, fetchFrom, toFetch, isIncremental ? desc.parentSessionId : null,
+                        task = new LocalSyncTask(ctx, desc, address, fetchFrom, toFetch, isIncremental ? desc.parentSessionId : null,
                                                  true, false, previewKind);
                     }
                     else
                     {
-                        task = new AsymmetricRemoteSyncTask(desc, address, fetchFrom, toFetch, previewKind);
+                        task = new AsymmetricRemoteSyncTask(ctx, desc, address, fetchFrom, toFetch, previewKind);
                     }
                     syncTasks.add(task);
 
@@ -457,14 +474,14 @@ public class RepairJob extends AsyncFuture<RepairResult> implements Runnable
             }
         }
         logger.info("Created {} optimised sync tasks based on {} merkle tree responses for {} (took: {}ms)",
-                    syncTasks.size(), trees.size(), desc.parentSessionId, currentTimeMillis() - startedAt);
+                    syncTasks.size(), trees.size(), desc.parentSessionId, ctx.clock().currentTimeMillis() - startedAt);
         logger.trace("Optimised sync tasks for {}: {}", desc.parentSessionId, syncTasks);
         return syncTasks;
     }
 
     private String getDC(InetAddressAndPort address)
     {
-        return DatabaseDescriptor.getEndpointSnitch().getDatacenter(address);
+        return ctx.snitch().getDatacenter(address);
     }
 
     /**
@@ -479,8 +496,8 @@ public class RepairJob extends AsyncFuture<RepairResult> implements Runnable
         String message = String.format("Requesting merkle trees for %s (to %s)", desc.columnFamily, endpoints);
         logger.info("{} {}", session.previewKind.logPrefix(desc.sessionId), message);
         Tracing.traceRepair(message);
-        int nowInSec = getNowInSeconds();
-        List<Future<TreeResponse>> tasks = new ArrayList<>(endpoints.size());
+        long nowInSec = getNowInSeconds();
+        List<ValidationTask> tasks = new ArrayList<>(endpoints.size());
         for (InetAddressAndPort endpoint : endpoints)
         {
             ValidationTask task = newValidationTask(endpoint, nowInSec);
@@ -500,7 +517,7 @@ public class RepairJob extends AsyncFuture<RepairResult> implements Runnable
         String message = String.format("Requesting merkle trees for %s (to %s)", desc.columnFamily, endpoints);
         logger.info("{} {}", session.previewKind.logPrefix(desc.sessionId), message);
         Tracing.traceRepair(message);
-        int nowInSec = getNowInSeconds();
+        long nowInSec = getNowInSeconds();
         List<Future<TreeResponse>> tasks = new ArrayList<>(endpoints.size());
 
         Queue<InetAddressAndPort> requests = new LinkedList<>(endpoints);
@@ -515,7 +532,7 @@ public class RepairJob extends AsyncFuture<RepairResult> implements Runnable
             final InetAddressAndPort nextAddress = requests.poll();
             final ValidationTask nextTask = newValidationTask(nextAddress, nowInSec);
             tasks.add(nextTask);
-            currentTask.addCallback(new FutureCallback<TreeResponse>()
+            currentTask.addCallback(new FutureCallback<>()
             {
                 public void onSuccess(TreeResponse result)
                 {
@@ -543,7 +560,7 @@ public class RepairJob extends AsyncFuture<RepairResult> implements Runnable
         String message = String.format("Requesting merkle trees for %s (to %s)", desc.columnFamily, endpoints);
         logger.info("{} {}", session.previewKind.logPrefix(desc.sessionId), message);
         Tracing.traceRepair(message);
-        int nowInSec = getNowInSeconds();
+        long nowInSec = getNowInSeconds();
         List<Future<TreeResponse>> tasks = new ArrayList<>(endpoints.size());
 
         Map<String, Queue<InetAddressAndPort>> requestsByDatacenter = new HashMap<>();
@@ -568,7 +585,7 @@ public class RepairJob extends AsyncFuture<RepairResult> implements Runnable
                 final InetAddressAndPort nextAddress = requests.poll();
                 final ValidationTask nextTask = newValidationTask(nextAddress, nowInSec);
                 tasks.add(nextTask);
-                currentTask.addCallback(new FutureCallback<TreeResponse>()
+                currentTask.addCallback(new FutureCallback<>()
                 {
                     public void onSuccess(TreeResponse result)
                     {
@@ -588,9 +605,9 @@ public class RepairJob extends AsyncFuture<RepairResult> implements Runnable
         return FutureCombiner.allOf(tasks);
     }
 
-    private ValidationTask newValidationTask(InetAddressAndPort endpoint, int nowInSec)
+    private ValidationTask newValidationTask(InetAddressAndPort endpoint, long nowInSec)
     {
-        ValidationTask task = new ValidationTask(desc, endpoint, nowInSec, session.previewKind);
+        ValidationTask task = new ValidationTask(session.ctx, desc, endpoint, nowInSec, session.previewKind);
         validationTasks.add(task);
         return task;
     }
