@@ -17,8 +17,7 @@
  */
 package org.apache.cassandra.repair;
 
-import java.net.InetAddress;
-
+import org.apache.cassandra.locator.InetAddressAndPort;
 import org.apache.cassandra.utils.MerkleTrees;
 
 /**
@@ -26,10 +25,10 @@ import org.apache.cassandra.utils.MerkleTrees;
  */
 public class TreeResponse
 {
-    public final InetAddress endpoint;
+    public final InetAddressAndPort endpoint;
     public final MerkleTrees trees;
 
-    public TreeResponse(InetAddress endpoint, MerkleTrees trees)
+    public TreeResponse(InetAddressAndPort endpoint, MerkleTrees trees)
     {
         this.endpoint = endpoint;
         this.trees = trees;
