@@ -21,6 +21,7 @@ import java.lang.reflect.Field;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.LinkedHashMap;
 import java.util.Map;
 import java.util.Set;
@@ -147,8 +148,58 @@ public final class MemtableParams
         if (!memtableConfigurations.containsKey(DEFAULT_CONFIGURATION_KEY))
             configs.put(DEFAULT_CONFIGURATION_KEY, DEFAULT_CONFIGURATION);
 
-        for (Map.Entry<String, InheritingClass> en : memtableConfigurations.entrySet())
-            configs.put(en.getKey(), en.getValue().resolve(configs));
+        Map<String, InheritingClass> inheritingClasses = new LinkedHashMap<>();
+
+        for (Map.Entry<String, InheritingClass> entry : memtableConfigurations.entrySet())
+        {
+            if (entry.getValue().inherits != null)
+            {
+                if (entry.getKey().equals(entry.getValue().inherits))
+                    throw new ConfigurationException(String.format("Configuration entry %s can not inherit itself.", entry.getKey()));
+
+                if (memtableConfigurations.get(entry.getValue().inherits) == null && !entry.getValue().inherits.equals(DEFAULT_CONFIGURATION_KEY))
+                    throw new ConfigurationException(String.format("Configuration entry %s inherits non-existing entry %s.",
+                                                                   entry.getKey(), entry.getValue().inherits));
+
+                inheritingClasses.put(entry.getKey(), entry.getValue());
+            }
+            else
+                configs.put(entry.getKey(), entry.getValue().resolve(configs));
+        }
+
+        for (Map.Entry<String, InheritingClass> inheritingEntry : inheritingClasses.entrySet())
+        {
+            String inherits = inheritingEntry.getValue().inherits;
+            while (inherits != null)
+            {
+                InheritingClass nextInheritance = inheritingClasses.get(inherits);
+                if (nextInheritance == null)
+                    inherits = null;
+                else
+                    inherits = nextInheritance.inherits;
+
+                if (inherits != null && inherits.equals(inheritingEntry.getKey()))
+                    throw new ConfigurationException(String.format("Detected loop when processing key %s", inheritingEntry.getKey()));
+            }
+        }
+
+        while (!inheritingClasses.isEmpty())
+        {
+            Set<String> forRemoval = new HashSet<>();
+            for (Map.Entry<String, InheritingClass> inheritingEntry : inheritingClasses.entrySet())
+            {
+                if (configs.get(inheritingEntry.getValue().inherits) != null)
+                {
+                    configs.put(inheritingEntry.getKey(), inheritingEntry.getValue().resolve(configs));
+                    forRemoval.add(inheritingEntry.getKey());
+                }
+            }
+
+            assert !forRemoval.isEmpty();
+
+            for (String toRemove : forRemoval)
+                inheritingClasses.remove(toRemove);
+        }
 
         return ImmutableMap.copyOf(configs);
     }
