@@ -27,7 +27,10 @@ import java.util.regex.Pattern;
 import com.google.common.collect.ImmutableSet;
 import com.google.common.collect.Sets;
 
+import org.apache.cassandra.auth.AuthKeyspace;
 import org.apache.cassandra.db.Digest;
+import org.apache.cassandra.db.SystemKeyspace;
+import org.apache.cassandra.tracing.TraceKeyspace;
 
 /**
  * When adding new String keyspace names here, double check if it needs to be added to PartitionDenylist.canDenylistKeyspace
@@ -47,6 +50,8 @@ public final class SchemaConstants
 
     public static final String VIRTUAL_VIEWS = "system_views";
 
+    public static final String DUMMY_KEYSPACE_OR_TABLE_NAME = "--dummy--";
+
     /* system keyspace names (the ones with LocalStrategy replication strategy) */
     public static final Set<String> LOCAL_SYSTEM_KEYSPACE_NAMES =
         ImmutableSet.of(SYSTEM_KEYSPACE_NAME, SCHEMA_KEYSPACE_NAME);
@@ -134,4 +139,28 @@ public final class SchemaConstants
     {
         return Sets.union(Sets.union(LOCAL_SYSTEM_KEYSPACE_NAMES, REPLICATED_SYSTEM_KEYSPACE_NAMES), VIRTUAL_SYSTEM_KEYSPACE_NAMES);
     }
+
+    /**
+     * Returns the set of local and replicated system keyspace names
+     * @return all local and replicated system keyspace names
+     */
+    public static Set<String> getLocalAndReplicatedSystemKeyspaceNames()
+    {
+        return Sets.union(LOCAL_SYSTEM_KEYSPACE_NAMES, REPLICATED_SYSTEM_KEYSPACE_NAMES);
+    }
+    
+    /**
+     * Returns the set of all local and replicated system table names
+     * @return all local and replicated system table names
+     */
+    public static Set<String> getLocalAndReplicatedSystemTableNames()
+    {
+        return ImmutableSet.<String>builder()
+                           .addAll(SystemKeyspace.TABLE_NAMES)
+                           .addAll(SchemaKeyspaceTables.ALL)
+                           .addAll(TraceKeyspace.TABLE_NAMES)
+                           .addAll(AuthKeyspace.TABLE_NAMES)
+                           .addAll(SystemDistributedKeyspace.TABLE_NAMES)
+                           .build();
+    }
 }
