@@ -133,4 +133,10 @@ public class SimpleDateSerializer extends TypeSerializer<Integer>
     {
         return Integer.class;
     }
+
+    @Override
+    public boolean shouldQuoteCQLLiterals()
+    {
+        return true;
+    }
 }
