@@ -40,6 +40,8 @@ import org.apache.cassandra.transport.Dispatcher;
 import org.apache.cassandra.transport.Server;
 import org.apache.cassandra.utils.NativeLibrary;
 
+import static org.apache.cassandra.config.CassandraRelevantProperties.NATIVE_EPOLL_ENABLED;
+
 /**
  * Handles native transport server lifecycle and associated resources. Lazily initialized.
  */
@@ -137,7 +139,12 @@ public class NativeTransportService
      */
     public void stop()
     {
-        servers.forEach(Server::stop);
+        stop(false);
+    }
+
+    public void stop(boolean force)
+    {
+        servers.forEach((s) -> s.stop(force));
     }
 
     /**
@@ -160,7 +167,7 @@ public class NativeTransportService
      */
     public static boolean useEpoll()
     {
-        final boolean enableEpoll = Boolean.parseBoolean(System.getProperty("cassandra.native.epoll.enabled", "true"));
+        final boolean enableEpoll = NATIVE_EPOLL_ENABLED.getBoolean();
 
         if (enableEpoll && !Epoll.isAvailable() && NativeLibrary.osType == NativeLibrary.OSType.LINUX)
             logger.warn("epoll not available", Epoll.unavailabilityCause());
