@@ -50,7 +50,7 @@ public class MultiPartitionPager<T extends SinglePartitionReadQuery> implements
     private final SinglePartitionPager[] pagers;
     private final DataLimits limit;
 
-    private final int nowInSec;
+    private final long nowInSec;
 
     private int remaining;
     private int current;
@@ -88,7 +88,7 @@ public class MultiPartitionPager<T extends SinglePartitionReadQuery> implements
 
     private MultiPartitionPager(SinglePartitionPager[] pagers,
                                 DataLimits limit,
-                                int nowInSec,
+                                long nowInSec,
                                 int remaining,
                                 int current)
     {
@@ -156,7 +156,6 @@ public class MultiPartitionPager<T extends SinglePartitionReadQuery> implements
         return new PagersIterator(toQuery, consistency, clientState, null, requestTime);
     }
 
-    @SuppressWarnings("resource") // iter closed via countingIter
     public PartitionIterator fetchPageInternal(int pageSize, ReadExecutionController executionController) throws RequestValidationException, RequestExecutionException
     {
         int toQuery = Math.min(remaining, pageSize);
