@@ -20,16 +20,22 @@ package org.apache.cassandra.streaming;
 import java.io.EOFException;
 import java.net.SocketTimeoutException;
 import java.nio.channels.ClosedChannelException;
+import java.nio.file.FileStore;
 import java.util.ArrayList;
 import java.util.Collection;
+import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
+import java.util.IdentityHashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.Objects;
 import java.util.Set;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ConcurrentMap;
 import java.util.concurrent.TimeUnit;
+import java.util.function.Function;
+import java.util.stream.Collectors;
 
 import javax.annotation.Nullable;
 
@@ -40,26 +46,30 @@ import com.google.common.collect.Lists;
 import com.google.common.collect.Multimap;
 import com.google.common.collect.Sets;
 
+import io.netty.channel.Channel;
+import io.netty.util.concurrent.Future; //checkstyle: permit this import
+
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import io.netty.channel.Channel;
-import io.netty.util.concurrent.Future; //checkstyle: permit this import
 import org.apache.cassandra.concurrent.ScheduledExecutors;
 import org.apache.cassandra.config.DatabaseDescriptor;
 import org.apache.cassandra.db.ColumnFamilyStore;
+import org.apache.cassandra.db.Directories;
 import org.apache.cassandra.db.Keyspace;
+import org.apache.cassandra.db.compaction.CompactionManager;
+import org.apache.cassandra.db.compaction.CompactionStrategyManager;
 import org.apache.cassandra.db.lifecycle.TransactionAlreadyCompletedException;
 import org.apache.cassandra.dht.OwnedRanges;
 import org.apache.cassandra.dht.Range;
 import org.apache.cassandra.dht.Token;
-import org.apache.cassandra.gms.EndpointState;
-import org.apache.cassandra.gms.IEndpointStateChangeSubscriber;
+import org.apache.cassandra.io.util.File;
 import org.apache.cassandra.locator.InetAddressAndPort;
 import org.apache.cassandra.locator.RangesAtEndpoint;
 import org.apache.cassandra.locator.Replica;
 import org.apache.cassandra.metrics.StreamingMetrics;
 import org.apache.cassandra.schema.TableId;
+import org.apache.cassandra.service.ActiveRepairService;
 import org.apache.cassandra.streaming.async.StreamingMultiplexedChannel;
 import org.apache.cassandra.service.StorageService;
 import org.apache.cassandra.streaming.messages.*;
@@ -70,6 +80,7 @@ import org.apache.cassandra.utils.TimeUUID;
 import org.apache.cassandra.utils.concurrent.FutureCombiner;
 
 import static com.google.common.collect.Iterables.all;
+import static org.apache.cassandra.config.CassandraRelevantProperties.CASSANDRA_STREAMING_DEBUG_STACKTRACE_LIMIT;
 import static org.apache.cassandra.utils.Clock.Global.nanoTime;
 import static org.apache.cassandra.locator.InetAddressAndPort.hostAddressAndPort;
 import static org.apache.cassandra.utils.FBUtilities.getBroadcastAddressAndPort;
@@ -149,9 +160,10 @@ import static org.apache.cassandra.utils.FBUtilities.getBroadcastAddressAndPort;
  * (via {@link org.apache.cassandra.net.MessagingService}, while the actual files themselves are sent by a special
  * "streaming" connection type. See {@link StreamingMultiplexedChannel} for details. Because of the asynchronous
  */
-public class StreamSession implements IEndpointStateChangeSubscriber
+public class StreamSession
 {
     private static final Logger logger = LoggerFactory.getLogger(StreamSession.class);
+    private static final int DEBUG_STACKTRACE_LIMIT = CASSANDRA_STREAMING_DEBUG_STACKTRACE_LIMIT.getInt();
 
     public enum PrepareDirection { SEND, ACK }
 
@@ -199,6 +211,8 @@ public class StreamSession implements IEndpointStateChangeSubscriber
     private final TimeUUID pendingRepair;
     private final PreviewKind previewKind;
 
+    public String failureReason;
+
 /**
  * State Transition:
  *
@@ -281,7 +295,15 @@ public class StreamSession implements IEndpointStateChangeSubscriber
 
     public StreamOperation streamOperation()
     {
-        return streamResult == null ? null : streamResult.streamOperation;
+        if (streamResult == null)
+        {
+            logger.warn("StreamResultFuture not initialized {} {}", channel.connectedTo(), isFollower ? "follower" : "initiator");
+            return null;
+        }
+        else
+        {
+            return streamResult.streamOperation;
+        }
     }
 
     public StreamOperation getStreamOperation()
@@ -503,6 +525,11 @@ public class StreamSession implements IEndpointStateChangeSubscriber
     }
 
     private Future<?> closeSession(State finalState)
+    {
+        return closeSession(finalState, null);
+    }
+
+    private Future<?> closeSession(State finalState, String failureReason)
     {
         // Keep a separate lock on the closeFuture so that we create it once and only once.
         // Cannot use the StreamSession monitor here as StreamDeserializingTask/StreamSession.messageReceived
@@ -516,9 +543,13 @@ public class StreamSession implements IEndpointStateChangeSubscriber
             closeFuture = ScheduledExecutors.nonPeriodicTasks.submit(() -> {
                 synchronized (this) {
                     state(finalState);
+                    //this refers to StreamInfo
+                    this.failureReason = failureReason;
 
                     sink.onClose(peer);
-                    streamResult.handleSessionComplete(this);
+                    // closed before init?
+                    if (streamResult != null)
+                        streamResult.handleSessionComplete(this);
                 }}).flatMap(ignore -> {
                     List<Future<?>> futures = new ArrayList<>();
                     // ensure aborting the tasks do not happen on the network IO thread (read: netty event loop)
@@ -679,7 +710,6 @@ public class StreamSession implements IEndpointStateChangeSubscriber
             if (state.finalState)
             {
                 logger.debug("[Stream #{}] Socket closed after session completed with state {}", planId(), state);
-
                 return null;
             }
             else
@@ -688,8 +718,7 @@ public class StreamSession implements IEndpointStateChangeSubscriber
                              planId(),
                              peer.getHostAddressAndPort(),
                              e);
-
-                return closeSession(State.FAILED);
+                return closeSession(State.FAILED, "Failed because there was an " + e.getClass().getCanonicalName() + " with state=" + state.name());
             }
         }
         else if (e instanceof TransactionAlreadyCompletedException && isFailedOrAborted())
@@ -709,8 +738,9 @@ public class StreamSession implements IEndpointStateChangeSubscriber
             state(State.FAILED); // make sure subsequent error handling sees the session in a final state
             sendControlMessage(new SessionFailedMessage()).awaitUninterruptibly();
         }
-
-        return closeSession(State.FAILED);
+        StringBuilder failureReason = new StringBuilder("Failed because of an unknown exception\n");
+        boundStackTrace(e, DEBUG_STACKTRACE_LIMIT, failureReason);
+        return closeSession(State.FAILED, failureReason.toString());
     }
 
     private void logError(Throwable e)
@@ -764,6 +794,8 @@ public class StreamSession implements IEndpointStateChangeSubscriber
     @VisibleForTesting
     public void prepareAsync(Collection<StreamRequest> requests, Collection<StreamSummary> summaries)
     {
+        if (StreamOperation.REPAIR == streamOperation())
+            checkAvailableDiskSpaceAndCompactions(summaries);
         processStreamRequests(requests);
         for (StreamSummary summary : summaries)
             prepareReceiving(summary);
@@ -791,6 +823,8 @@ public class StreamSession implements IEndpointStateChangeSubscriber
 
     private void prepareSynAck(PrepareSynAckMessage msg)
     {
+        if (StreamOperation.REPAIR == streamOperation())
+            checkAvailableDiskSpaceAndCompactions(msg.summaries);
         if (!msg.summaries.isEmpty())
         {
             for (StreamSummary summary : msg.summaries)
@@ -814,6 +848,163 @@ public class StreamSession implements IEndpointStateChangeSubscriber
         startStreamingFiles(PrepareDirection.ACK);
     }
 
+    /**
+     * In the case where we have an error checking disk space we allow the Operation to continue.
+     * In the case where we do _not_ have available space, this method raises a RTE.
+     * TODO: Consider revising this to returning a boolean and allowing callers upstream to handle that.
+     */
+    private void checkAvailableDiskSpaceAndCompactions(Collection<StreamSummary> summaries)
+    {
+        if (DatabaseDescriptor.getSkipStreamDiskSpaceCheck())
+            return;
+
+        boolean hasAvailableSpace = true;
+
+        try
+        {
+            hasAvailableSpace = checkAvailableDiskSpaceAndCompactions(summaries, planId(), peer.getHostAddress(true), pendingRepair != null);
+        }
+        catch (Exception e)
+        {
+            logger.error("[Stream #{}] Could not check available disk space and compactions for {}, summaries = {}", planId(), this, summaries, e);
+        }
+        if (!hasAvailableSpace)
+            throw new RuntimeException(String.format("Not enough disk space for stream %s), summaries=%s", this, summaries));
+    }
+
+    /**
+     * Makes sure that we expect to have enough disk space available for the new streams, taking into consideration
+     * the ongoing compactions and streams.
+     */
+    @VisibleForTesting
+    public static boolean checkAvailableDiskSpaceAndCompactions(Collection<StreamSummary> summaries,
+                                                                @Nullable TimeUUID planId,
+                                                                @Nullable String remoteAddress,
+                                                                boolean isForIncremental)
+    {
+        Map<TableId, Long> perTableIdIncomingBytes = new HashMap<>();
+        Map<TableId, Integer> perTableIdIncomingFiles = new HashMap<>();
+        long newStreamTotal = 0;
+        for (StreamSummary summary : summaries)
+        {
+            perTableIdIncomingFiles.merge(summary.tableId, summary.files, Integer::sum);
+            perTableIdIncomingBytes.merge(summary.tableId, summary.totalSize, Long::sum);
+            newStreamTotal += summary.totalSize;
+        }
+        if (perTableIdIncomingBytes.isEmpty() || newStreamTotal == 0)
+            return true;
+
+        return checkDiskSpace(perTableIdIncomingBytes, planId, Directories::getFileStore) &&
+               checkPendingCompactions(perTableIdIncomingBytes, perTableIdIncomingFiles, planId, remoteAddress, isForIncremental, newStreamTotal);
+    }
+
+    @VisibleForTesting
+    static boolean checkDiskSpace(Map<TableId, Long> perTableIdIncomingBytes,
+                                  TimeUUID planId,
+                                  Function<File, FileStore> fileStoreMapper)
+    {
+        Map<FileStore, Long> newStreamBytesToWritePerFileStore = new HashMap<>();
+        Set<FileStore> allFileStores = new HashSet<>();
+        // Sum up the incoming bytes per file store - we assume that the stream is evenly distributed over the writable
+        // file stores for the table.
+        for (Map.Entry<TableId, Long> entry : perTableIdIncomingBytes.entrySet())
+        {
+            ColumnFamilyStore cfs = ColumnFamilyStore.getIfExists(entry.getKey());
+            if (cfs == null || perTableIdIncomingBytes.get(entry.getKey()) == 0)
+                continue;
+
+            Set<FileStore> allWriteableFileStores = cfs.getDirectories().allFileStores(fileStoreMapper);
+            if (allWriteableFileStores.isEmpty())
+            {
+                logger.error("[Stream #{}] Could not get any writeable FileStores for {}.{}", planId, cfs.getKeyspaceName(), cfs.getTableName());
+                continue;
+            }
+            allFileStores.addAll(allWriteableFileStores);
+            long totalBytesInPerFileStore = entry.getValue() / allWriteableFileStores.size();
+            for (FileStore fs : allWriteableFileStores)
+                newStreamBytesToWritePerFileStore.merge(fs, totalBytesInPerFileStore, Long::sum);
+        }
+        Map<FileStore, Long> totalCompactionWriteRemaining = Directories.perFileStore(CompactionManager.instance.active.estimatedRemainingWriteBytes(),
+                                                                                      fileStoreMapper);
+        long totalStreamRemaining = StreamManager.instance.getTotalRemainingOngoingBytes();
+        long totalBytesStreamRemainingPerFileStore = totalStreamRemaining / Math.max(1, allFileStores.size());
+        Map<FileStore, Long> allWriteData = new HashMap<>();
+        for (Map.Entry<FileStore, Long> fsBytes : newStreamBytesToWritePerFileStore.entrySet())
+            allWriteData.put(fsBytes.getKey(), fsBytes.getValue() +
+                                               totalBytesStreamRemainingPerFileStore +
+                                               totalCompactionWriteRemaining.getOrDefault(fsBytes.getKey(), 0L));
+
+        if (!Directories.hasDiskSpaceForCompactionsAndStreams(allWriteData))
+        {
+            logger.error("[Stream #{}] Not enough disk space to stream {} to {} (stream ongoing remaining={}, compaction ongoing remaining={}, all ongoing writes={})",
+                         planId,
+                         newStreamBytesToWritePerFileStore,
+                         perTableIdIncomingBytes.keySet().stream()
+                                                .map(ColumnFamilyStore::getIfExists).filter(Objects::nonNull)
+                                                .map(cfs -> cfs.getKeyspaceName() + '.' + cfs.name)
+                                                .collect(Collectors.joining(",")),
+                         totalStreamRemaining,
+                         totalCompactionWriteRemaining,
+                         allWriteData);
+            return false;
+        }
+        return true;
+    }
+
+    @VisibleForTesting
+    static boolean checkPendingCompactions(Map<TableId, Long> perTableIdIncomingBytes,
+                                           Map<TableId, Integer> perTableIdIncomingFiles,
+                                           TimeUUID planId, String remoteAddress,
+                                           boolean isForIncremental,
+                                           long newStreamTotal)
+    {
+
+        int pendingCompactionsBeforeStreaming = 0;
+        int pendingCompactionsAfterStreaming = 0;
+        List<String> tables = new ArrayList<>(perTableIdIncomingFiles.size());
+        for (Keyspace ks : Keyspace.all())
+        {
+            Map<ColumnFamilyStore, TableId> cfStreamed = perTableIdIncomingBytes.keySet().stream()
+                                                                                .filter(ks::hasColumnFamilyStore)
+                                                                                .collect(Collectors.toMap(ks::getColumnFamilyStore, Function.identity()));
+            for (ColumnFamilyStore cfs : ks.getColumnFamilyStores())
+            {
+                CompactionStrategyManager csm = cfs.getCompactionStrategyManager();
+                int tasksOther = csm.getEstimatedRemainingTasks();
+                int tasksStreamed = tasksOther;
+                if (cfStreamed.containsKey(cfs))
+                {
+                    TableId tableId = cfStreamed.get(cfs);
+                    tasksStreamed = csm.getEstimatedRemainingTasks(perTableIdIncomingFiles.get(tableId),
+                                                                   perTableIdIncomingBytes.get(tableId),
+                                                                   isForIncremental);
+                    tables.add(String.format("%s.%s", cfs.getKeyspaceName(), cfs.name));
+                }
+                pendingCompactionsBeforeStreaming += tasksOther;
+                pendingCompactionsAfterStreaming += tasksStreamed;
+            }
+        }
+        Collections.sort(tables);
+        int pendingThreshold = ActiveRepairService.instance().getRepairPendingCompactionRejectThreshold();
+        if (pendingCompactionsAfterStreaming > pendingThreshold)
+        {
+            logger.error("[Stream #{}] Rejecting incoming files based on pending compactions calculation " +
+                         "pendingCompactionsBeforeStreaming={} pendingCompactionsAfterStreaming={} pendingThreshold={} remoteAddress={}",
+                         planId, pendingCompactionsBeforeStreaming, pendingCompactionsAfterStreaming, pendingThreshold, remoteAddress);
+            return false;
+        }
+
+        long newStreamFiles = perTableIdIncomingFiles.values().stream().mapToInt(i -> i).sum();
+
+        logger.info("[Stream #{}] Accepting incoming files newStreamTotalSSTables={} newStreamTotalBytes={} " +
+                    "pendingCompactionsBeforeStreaming={} pendingCompactionsAfterStreaming={} pendingThreshold={} remoteAddress={} " +
+                    "streamedTables=\"{}\"",
+                    planId, newStreamFiles, newStreamTotal,
+                    pendingCompactionsBeforeStreaming, pendingCompactionsAfterStreaming, pendingThreshold, remoteAddress,
+                    String.join(",", tables));
+        return true;
+    }
+
     private void processStreamRequests(Collection<StreamRequest> requests)
     {
         List<StreamRequest> rejectedRequests = new ArrayList<>();
@@ -861,7 +1052,7 @@ public class StreamSession implements IEndpointStateChangeSubscriber
         StreamTransferTask task = transfers.get(message.header.tableId);
         if (task != null)
         {
-            task.scheduleTimeout(message.header.sequenceNumber, 12, TimeUnit.HOURS);
+            task.scheduleTimeout(message.header.sequenceNumber, DatabaseDescriptor.getStreamTransferTaskTimeout().toMilliseconds(), TimeUnit.MILLISECONDS);
         }
     }
 
@@ -993,7 +1184,9 @@ public class StreamSession implements IEndpointStateChangeSubscriber
     public synchronized void sessionFailed()
     {
         logger.error("[Stream #{}] Remote peer {} failed stream session.", planId(), peer.toString());
-        closeSession(State.FAILED);
+        StringBuilder stringBuilder = new StringBuilder();
+        stringBuilder.append("Remote peer ").append(peer).append(" failed stream session");
+        closeSession(State.FAILED, stringBuilder.toString());
     }
 
     /**
@@ -1002,7 +1195,7 @@ public class StreamSession implements IEndpointStateChangeSubscriber
     public synchronized void sessionTimeout()
     {
         logger.error("[Stream #{}] timeout with {}.", planId(), peer.toString());
-        closeSession(State.FAILED);
+        closeSession(State.FAILED, "Session timed out");
     }
 
     /**
@@ -1016,7 +1209,7 @@ public class StreamSession implements IEndpointStateChangeSubscriber
         List<StreamSummary> transferSummaries = Lists.newArrayList();
         for (StreamTask transfer : transfers.values())
             transferSummaries.add(transfer.getSummary());
-        return new SessionInfo(channel.peer(), index, channel.connectedTo(), receivingSummaries, transferSummaries, state);
+        return new SessionInfo(channel.peer(), index, channel.connectedTo(), receivingSummaries, transferSummaries, state, failureReason);
     }
 
     public synchronized void taskCompleted(StreamReceiveTask completedTask)
@@ -1031,18 +1224,6 @@ public class StreamSession implements IEndpointStateChangeSubscriber
         maybeCompleted();
     }
 
-    public void onRemove(InetAddressAndPort endpoint)
-    {
-        logger.error("[Stream #{}] Session failed because remote peer {} has left.", planId(), peer.toString());
-        closeSession(State.FAILED);
-    }
-
-    public void onRestart(InetAddressAndPort endpoint, EndpointState epState)
-    {
-        logger.error("[Stream #{}] Session failed because remote peer {} was restarted.", planId(), peer.toString());
-        closeSession(State.FAILED);
-    }
-
     private void completePreview()
     {
         try
@@ -1211,4 +1392,53 @@ public class StreamSession implements IEndpointStateChangeSubscriber
             logger.error("[Stream #{}] Error aborting stream session with peer {}", planId(), peer);
         }
     }
+
+    @Override
+    public String toString()
+    {
+        return "StreamSession{" +
+               "streamOperation=" + streamOperation +
+               ", peer=" + peer +
+               ", channel=" + channel +
+               ", requests=" + requests +
+               ", transfers=" + transfers +
+               ", isFollower=" + isFollower +
+               ", pendingRepair=" + pendingRepair +
+               ", previewKind=" + previewKind +
+               ", state=" + state +
+               '}';
+    }
+
+    public static StringBuilder boundStackTrace(Throwable e, int limit, StringBuilder out)
+    {
+        Set<Throwable> visited = Collections.newSetFromMap(new IdentityHashMap<>());
+        return boundStackTrace(e, limit, limit, visited, out);
+    }
+
+    public static StringBuilder boundStackTrace(Throwable e, int limit, int counter, Set<Throwable> visited, StringBuilder out)
+    {
+        if (e == null)
+            return out;
+
+        if (!visited.add(e))
+            return out.append("[CIRCULAR REFERENCE: ").append(e.getClass().getName()).append(": ").append(e.getMessage()).append("]").append('\n');
+        visited.add(e);
+
+        StackTraceElement[] stackTrace = e.getStackTrace();
+        out.append(e.getClass().getName() + ": " + e.getMessage()).append('\n');
+
+        // When dealing with the leaf, ignore how many stack traces were already written, and allow the max.
+        // This is here as the leaf tends to show where the issue started, so tends to be impactful for debugging
+        if (e.getCause() == null)
+            counter = limit;
+
+        for (int i = 0, size = Math.min(e.getStackTrace().length, limit); i < size && counter > 0; i++)
+        {
+            out.append('\t').append(stackTrace[i]).append('\n');
+            counter--;
+        }
+
+        boundStackTrace(e.getCause(), limit, counter, visited, out);
+        return out;
+    }
 }
