@@ -37,6 +37,16 @@ public class Decommission extends NodeToolCmd
     {
         try
         {
+            if (probe.getStorageService().isDecommissioning())
+            {
+                probe.output().out.println("This node is still decommissioning.");
+                return;
+            }
+            if ("DECOMMISSIONED".equals(probe.getStorageService().getBootstrapState()))
+            {
+                probe.output().out.println("Node was already decommissioned.");
+                return;
+            }
             probe.decommission(force);
         } catch (InterruptedException e)
         {
@@ -46,4 +56,4 @@ public class Decommission extends NodeToolCmd
             throw new IllegalStateException("Unsupported operation: " + e.getMessage(), e);
         }
     }
-}
\ No newline at end of file
+}
