@@ -19,6 +19,8 @@ package org.apache.cassandra.utils;
 
 import com.google.common.base.Objects;
 
+import static com.google.common.base.Preconditions.checkNotNull;
+
 public class Interval<C, D>
 {
     public final C min;
@@ -27,6 +29,8 @@ public class Interval<C, D>
 
     public Interval(C min, C max, D data)
     {
+        checkNotNull(min, "min is null");
+        checkNotNull(max, "max is null");
         this.min = min;
         this.max = max;
         this.data = data;
@@ -65,35 +69,49 @@ public class Interval<C, D>
         return Objects.equal(min, that.min) && Objects.equal(max, that.max) && Objects.equal(data, that.data);
     }
 
-    private static final AsymmetricOrdering<Interval<Comparable, Object>, Comparable> minOrdering
-    = new AsymmetricOrdering<Interval<Comparable, Object>, Comparable>()
+    private static final AsymmetricOrdering<Interval<Comparable, Comparable>, Comparable> minOrdering
+    = new AsymmetricOrdering<Interval<Comparable, Comparable>, Comparable>()
     {
-        public int compareAsymmetric(Interval<Comparable, Object> left, Comparable right)
+        public int compareAsymmetric(Interval<Comparable, Comparable> left, Comparable right)
         {
             return left.min.compareTo(right);
         }
 
-        public int compare(Interval<Comparable, Object> i1, Interval<Comparable, Object> i2)
+        public int compare(Interval<Comparable, Comparable> i1, Interval<Comparable, Comparable> i2)
         {
-            return i1.min.compareTo(i2.min);
+            int cmpMin = i1.min.compareTo(i2.min);
+            if (cmpMin != 0)
+                return cmpMin;
+            int cmpMax = i1.max.compareTo(i2.max);
+            if (cmpMax != 0)
+                return cmpMax;
+            // Null is allowed if all data values are null otherwise NPE
+            return i1.data == i2.data ? 0 : i1.data.compareTo(i2.data);
         }
     };
 
-    private static final AsymmetricOrdering<Interval<Comparable, Object>, Comparable> maxOrdering
-    = new AsymmetricOrdering<Interval<Comparable, Object>, Comparable>()
+    private static final AsymmetricOrdering<Interval<Comparable, Comparable>, Comparable> maxOrdering
+    = new AsymmetricOrdering<Interval<Comparable, Comparable>, Comparable>()
     {
-        public int compareAsymmetric(Interval<Comparable, Object> left, Comparable right)
+        public int compareAsymmetric(Interval<Comparable, Comparable> left, Comparable right)
         {
             return left.max.compareTo(right);
         }
 
-        public int compare(Interval<Comparable, Object> i1, Interval<Comparable, Object> i2)
+        public int compare(Interval<Comparable, Comparable> i1, Interval<Comparable, Comparable> i2)
         {
-            return i1.max.compareTo(i2.max);
+            int cmpMax = i1.max.compareTo(i2.max);
+            if (cmpMax != 0)
+                return cmpMax;
+            int cmpMin = i1.min.compareTo(i2.min);
+            if (cmpMin != 0)
+                return cmpMin;
+            // Null is allowed if all data values are null otherwise NPE
+            return i1.data == i2.data ? 0 : i1.data.compareTo(i2.data);
         }
     };
 
-    private static final AsymmetricOrdering<Interval<Comparable, Object>, Comparable> reverseMaxOrdering = maxOrdering.reverse();
+    private static final AsymmetricOrdering<Interval<Comparable, Comparable>, Comparable> reverseMaxOrdering = maxOrdering.reverse();
 
     public static <C extends Comparable<? super C>, V> AsymmetricOrdering<Interval<C, V>, C> minOrdering()
     {
