@@ -53,6 +53,7 @@ import org.apache.cassandra.serializers.MarshalException;
 import org.apache.cassandra.serializers.TypeSerializer;
 
 import static java.util.concurrent.TimeUnit.MICROSECONDS;
+import static org.apache.cassandra.config.CassandraRelevantProperties.CASSANDRA_UNSAFE_TIME_UUID_NODE;
 import static org.apache.cassandra.config.CassandraRelevantProperties.DETERMINISM_UNSAFE_UUID_NODE;
 import static org.apache.cassandra.utils.ByteBufferUtil.EMPTY_BYTE_BUFFER;
 import static org.apache.cassandra.utils.Clock.Global.currentTimeMillis;
@@ -82,6 +83,7 @@ public class TimeUUID implements Serializable, Comparable<TimeUUID>
     public static final long MIN_CLOCK_SEQ_AND_NODE = 0x8080808080808080L;
     private static final long MAX_CLOCK_SEQ_AND_NODE = 0x7f7f7f7f7f7f7f7fL;
 
+    public static final long TIMEUUID_SIZE = ObjectSizes.measureDeep(new TimeUUID(10, 10));
 
     final long uuidTimestamp, lsb;
 
@@ -369,6 +371,16 @@ public class TimeUUID implements Serializable, Comparable<TimeUUID>
 
         private static final AtomicLong lastMicros = new AtomicLong();
 
+        public interface Factory<T extends TimeUUID>
+        {
+            T atUnixMicrosWithLsb(long unixMicros, long clockSeqAndNode);
+        }
+
+        public static <T extends TimeUUID> T nextTimeUUID(Factory<T> factory)
+        {
+            return factory.atUnixMicrosWithLsb(nextUnixMicros(), clockSeqAndNode);
+        }
+
         public static TimeUUID nextTimeUUID()
         {
             return atUnixMicrosWithLsb(nextUnixMicros(), clockSeqAndNode);
@@ -438,9 +450,8 @@ public class TimeUUID implements Serializable, Comparable<TimeUUID>
             if (DETERMINISM_UNSAFE_UUID_NODE.getBoolean())
                 return FBUtilities.getBroadcastAddressAndPort().addressBytes[3];
 
-            Long specified = Long.getLong("cassandra.unsafe.timeuuidnode");
-            if (specified != null)
-                return specified
+            if (CASSANDRA_UNSAFE_TIME_UUID_NODE.isPresent())
+                return CASSANDRA_UNSAFE_TIME_UUID_NODE.getLong()
                        ^ FBUtilities.getBroadcastAddressAndPort().addressBytes[3]
                        ^ (FBUtilities.getBroadcastAddressAndPort().addressBytes[2] << 8);
 
