@@ -52,6 +52,7 @@ import java.nio.ByteBuffer;
 
 import io.netty.util.concurrent.FastThreadLocal;
 import net.nicoulaj.compilecommand.annotations.Inline;
+import org.apache.cassandra.db.marshal.ValueAccessor;
 import org.apache.cassandra.io.util.DataInputPlus;
 import org.apache.cassandra.io.util.DataOutputPlus;
 
@@ -73,6 +74,21 @@ public class VIntCoding
 
     public static final int MAX_SIZE = 10;
 
+    /**
+     * Throw when attempting to decode a vint and the output type
+     * doesn't have enough space to fit the value that was decoded
+     */
+    public static class VIntOutOfRangeException extends RuntimeException
+    {
+        public final long value;
+
+        private VIntOutOfRangeException(long value)
+        {
+            super(value + " is out of range for a 32-bit integer");
+            this.value = value;
+        }
+    }
+
     public static long readUnsignedVInt(DataInput input) throws IOException
     {
         int firstByte = input.readByte();
@@ -101,14 +117,29 @@ public class VIntCoding
     }
 
     /**
+     * Read up to a 32-bit integer back, using the unsigned (no zigzag) encoding.
+     *
      * Note this method is the same as {@link #readUnsignedVInt(DataInput)},
      * except that we do *not* block if there are not enough bytes in the buffer
      * to reconstruct the value.
      *
-     * WARNING: this method is only safe for vints we know to be representable by a positive long value.
-     *
-     * @return -1 if there are not enough bytes in the input to read the value; else, the vint unsigned value.
+     * @throws VIntOutOfRangeException If the vint doesn't fit into a 32-bit integer
      */
+    public static int getUnsignedVInt32(ByteBuffer input, int readerIndex)
+    {
+        return checkedCast(getUnsignedVInt(input, readerIndex));
+    }
+
+    public static int getVInt32(ByteBuffer input, int readerIndex)
+    {
+        return checkedCast(decodeZigZag64(getUnsignedVInt(input, readerIndex)));
+    }
+
+    public static long getVInt(ByteBuffer input, int readerIndex)
+    {
+        return decodeZigZag64(getUnsignedVInt(input, readerIndex));
+    }
+
     public static long getUnsignedVInt(ByteBuffer input, int readerIndex)
     {
         return getUnsignedVInt(input, readerIndex, input.limit());
@@ -142,6 +173,55 @@ public class VIntCoding
         return retval;
     }
 
+    public static <V> int getUnsignedVInt32(V input, ValueAccessor<V> accessor, int readerIndex)
+    {
+        return checkedCast(getUnsignedVInt(input, accessor, readerIndex));
+    }
+
+    public static <V> int getVInt32(V input, ValueAccessor<V> accessor, int readerIndex)
+    {
+        return checkedCast(decodeZigZag64(getUnsignedVInt(input, accessor, readerIndex)));
+    }
+
+    public static <V> long getVInt(V input, ValueAccessor<V> accessor, int readerIndex)
+    {
+        return decodeZigZag64(getUnsignedVInt(input, accessor, readerIndex));
+    }
+
+    public static <V> long getUnsignedVInt(V input, ValueAccessor<V> accessor, int readerIndex)
+    {
+        return getUnsignedVInt(input, accessor, readerIndex, accessor.size(input));
+    }
+
+    public static <V> long getUnsignedVInt(V input, ValueAccessor<V> accessor, int readerIndex, int readerLimit)
+    {
+        if (readerIndex < 0)
+            throw new IllegalArgumentException("Reader index should be non-negative, but was " + readerIndex);
+
+        if (readerIndex >= readerLimit)
+            return -1;
+
+        int firstByte = accessor.getByte(input, readerIndex++);
+
+        //Bail out early if this is one byte, necessary or it fails later
+        if (firstByte >= 0)
+            return firstByte;
+
+        int size = numberOfExtraBytesToRead(firstByte);
+        if (readerIndex + size > readerLimit)
+            return -1;
+
+        long retval = firstByte & firstByteValueMask(size);
+        for (int ii = 0; ii < size; ii++)
+        {
+            byte b = accessor.getByte(input, readerIndex++);
+            retval <<= 8;
+            retval |= b & 0xff;
+        }
+
+        return retval;
+    }
+
     /**
      * Computes size of an unsigned vint that starts at readerIndex of the provided ByteBuf.
      *
@@ -165,6 +245,32 @@ public class VIntCoding
         return decodeZigZag64(readUnsignedVInt(input));
     }
 
+    /**
+     * Read up to a signed 32-bit integer back.
+     *
+     * Assumes the vint was written using {@link #writeVInt32(int, DataOutputPlus)} or similar
+     * that zigzag encodes the integer.
+     *
+     * @throws VIntOutOfRangeException If the vint doesn't fit into a 32-bit integer
+     */
+    public static int readVInt32(DataInput input) throws IOException
+    {
+        return checkedCast(decodeZigZag64(readUnsignedVInt(input)));
+    }
+
+    /**
+     * Read up to a 32-bit integer.
+     *
+     * This method assumes the original integer was written using {@link #writeUnsignedVInt32(int, DataOutputPlus)}
+     * or similar that doesn't zigzag encodes the vint.
+     *
+     * @throws VIntOutOfRangeException If the vint doesn't fit into a 32-bit integer
+     */
+    public static int readUnsignedVInt32(DataInput input) throws IOException
+    {
+        return checkedCast(readUnsignedVInt(input));
+    }
+
     // & this with the first byte to give the value part for a given extraBytesToRead encoded in the byte
     public static int firstByteValueMask(int extraBytesToRead)
     {
@@ -186,6 +292,13 @@ public class VIntCoding
         return Integer.numberOfLeadingZeros(~firstByte) - 24;
     }
 
+    /** @deprecated See CASSANDRA-18099 */
+    @Deprecated(since = "5.0")
+    public static void writeUnsignedVInt(int value, DataOutputPlus output) throws IOException
+    {
+        throw new UnsupportedOperationException("Use writeUnsignedVInt32/readUnsignedVInt32");
+    }
+
     @Inline
     public static void writeUnsignedVInt(long value, DataOutputPlus output) throws IOException
     {
@@ -200,7 +313,7 @@ public class VIntCoding
             int extraBytes = size - 1;
             long mask = (long)VIntCoding.encodeExtraBytesToRead(extraBytes) << 56;
             long register = (value << shift) | mask;
-            output.writeBytes(register, size);
+            output.writeMostSignificantBytes(register, size);
         }
         else if (size == 9)
         {
@@ -213,6 +326,18 @@ public class VIntCoding
         }
     }
 
+    public static void writeUnsignedVInt32(int value, DataOutputPlus output) throws IOException
+    {
+        writeUnsignedVInt((long)value, output);
+    }
+
+    /** @deprecated See CASSANDRA-18099 */
+    @Deprecated(since = "5.0")
+    public static void writeUnsignedVInt(int value, ByteBuffer output) throws IOException
+    {
+        throw new UnsupportedOperationException("Use writeUnsignedVInt32/getUnsignedVInt32");
+    }
+
     @Inline
     public static void writeUnsignedVInt(long value, ByteBuffer output)
     {
@@ -250,6 +375,74 @@ public class VIntCoding
         }
     }
 
+    @Inline
+    public static <V> int writeVInt(long value, V output, int offset, ValueAccessor<V> accessor)
+    {
+        return writeUnsignedVInt(encodeZigZag64(value), output, offset, accessor);
+    }
+
+    @Inline
+    public static  <V> int writeVInt32(int value, V output, int offset, ValueAccessor<V> accessor)
+    {
+        return writeVInt(value, output, offset, accessor);
+    }
+
+    @Inline
+    public static <V> int writeUnsignedVInt32(int value, V output, int offset, ValueAccessor<V> accessor)
+    {
+        return writeUnsignedVInt(value, output, offset, accessor);
+    }
+
+    @Inline
+    public static <V> int writeUnsignedVInt(long value, V output, int offset, ValueAccessor<V> accessor)
+    {
+        int size = VIntCoding.computeUnsignedVIntSize(value);
+        int written = 0;
+        if (size == 1)
+        {
+            written += accessor.putByte(output, offset, (byte) (value));
+        }
+        else if (size < 9)
+        {
+            if (accessor.remaining(output, offset) >= 8)
+            {
+                int shift = (8 - size) << 3;
+                int extraBytes = size - 1;
+                long mask = (long)VIntCoding.encodeExtraBytesToRead(extraBytes) << 56;
+                long register = (value << shift) | mask;
+                accessor.putLong(output, offset, register);
+                written += size;
+            }
+            else
+            {
+                written += accessor.putBytes(output, offset, VIntCoding.encodeUnsignedVInt(value, size), 0, size);
+            }
+        }
+        else if (size == 9)
+        {
+            written += accessor.putByte(output, offset, (byte) 0xFF);
+            written += accessor.putLong(output, offset + written, value);
+        }
+        else
+        {
+            throw new AssertionError();
+        }
+        return written;
+    }
+
+    @Inline
+    public static void writeUnsignedVInt32(int value, ByteBuffer output)
+    {
+        writeUnsignedVInt((long)value, output);
+    }
+
+    /** @deprecated See CASSANDRA-18099 */
+    @Deprecated(since = "5.0")
+    public static void writeVInt(int value, DataOutputPlus output) throws IOException
+    {
+        throw new UnsupportedOperationException("Use writeVInt32/readVInt32");
+    }
+
     @Inline
     public static void writeVInt(long value, DataOutputPlus output) throws IOException
     {
@@ -257,11 +450,30 @@ public class VIntCoding
     }
 
     @Inline
-    public static void writeVInt(long value, ByteBuffer output) throws IOException
+    public static void writeVInt32(int value, DataOutputPlus output) throws IOException
+    {
+        writeVInt((long)value, output);
+    }
+
+    /** @deprecated See CASSANDRA-18099 */
+    @Deprecated(since = "5.0")
+    public static void writeVInt(int value, ByteBuffer output)
+    {
+        throw new UnsupportedOperationException("Use writeVInt32/getVInt32");
+    }
+
+    @Inline
+    public static void writeVInt(long value, ByteBuffer output)
     {
         writeUnsignedVInt(encodeZigZag64(value), output);
     }
 
+    @Inline
+    public static void writeVInt32(int value, ByteBuffer output)
+    {
+        writeVInt((long)value, output);
+    }
+
     /**
      * @return a TEMPORARY THREAD LOCAL BUFFER containing the encoded bytes of the value
      * This byte[] must be discarded by the caller immediately, and synchronously
@@ -326,4 +538,12 @@ public class VIntCoding
         // the formula below is hand-picked to match the original 9 - ((magnitude - 1) / 7)
         return (639 - magnitude * 9) >> 6;
     }
+
+    public static int checkedCast(long value)
+    {
+        int result = (int)value;
+        if ((long)result != value)
+            throw new VIntOutOfRangeException(value);
+        return result;
+    }
 }
