@@ -17,11 +17,6 @@
  */
 package org.apache.cassandra.anttasks;
 
-import org.junit.Test;
-import org.reflections.Reflections;
-import org.reflections.scanners.Scanners;
-import org.reflections.util.ConfigurationBuilder;
-
 import java.io.File;
 import java.lang.annotation.Annotation;
 import java.lang.reflect.Method;
@@ -29,7 +24,10 @@ import java.lang.reflect.Modifier;
 import java.net.MalformedURLException;
 import java.net.URL;
 import java.nio.file.Paths;
+import java.util.ArrayDeque;
 import java.util.Arrays;
+import java.util.Deque;
+import java.util.HashSet;
 import java.util.List;
 import java.util.Optional;
 import java.util.Set;
@@ -37,6 +35,13 @@ import java.util.function.Predicate;
 import java.util.regex.Pattern;
 import java.util.stream.Stream;
 
+import org.apache.commons.lang3.function.Consumers;
+import org.junit.Test;
+
+import org.reflections.Reflections;
+import org.reflections.scanners.Scanners;
+import org.reflections.util.ConfigurationBuilder;
+
 import static java.util.stream.Collectors.toList;
 
 public class TestNameCheckTask
@@ -125,12 +130,11 @@ public class TestNameCheckTask
         if (normalize)
             stream = stream.map(this::normalize);
 
-        Pattern pattern = Pattern.compile(regex);
-        Predicate<String> patternPredicate = s -> !pattern.matcher(s).matches();
+        Predicate<String> patternPredicate = Predicate.not(Pattern.compile(regex).asMatchPredicate());
         List<String> classes = stream.map(Class::getCanonicalName)
                                      .distinct()
                                      .sorted()
-                                     .peek(verbose ? System.out::println : ignored -> {})
+                                     .peek(verbose ? System.out::println : Consumers.nop())
                                      .filter(patternPredicate)
                                      .collect(toList());
 
@@ -157,14 +161,21 @@ public class TestNameCheckTask
      */
     private Stream<? extends Class<?>> expand(Class<?> klass, Reflections reflections)
     {
-        Set<? extends Class<?>> subTypes = reflections.getSubTypesOf(klass);
-        if (subTypes == null || subTypes.isEmpty())
-            return Stream.of(klass);
-        Stream<? extends Class<?>> subs = subTypes.stream();
-        // assume we include if not abstract
-        if (!Modifier.isAbstract(klass.getModifiers()))
-            subs = Stream.concat(Stream.of(klass), subs);
-        return subs;
+        Set<Class<?>> concreteTypes = new HashSet<>();
+        Deque<Class<?>> typeStack = new ArrayDeque<>();
+        typeStack.push(klass);
+
+        while (!typeStack.isEmpty())
+        {
+            Class<?> type = typeStack.pop();
+
+            if (!Modifier.isAbstract(type.getModifiers()))
+                concreteTypes.add(type);
+
+            reflections.getSubTypesOf(type).forEach(typeStack::push);
+        }
+
+        return concreteTypes.stream();
     }
 
     public static void main(String[] args)
