@@ -163,7 +163,7 @@ public class ConnectionBurnTest
 
             public long serializedSize(byte[] payload, int version)
             {
-                return MessageGenerator.serializedSize(payload, version);
+                return MessageGenerator.serializedSize(payload);
             }
         };
 
