@@ -29,33 +29,33 @@ import org.apache.cassandra.distributed.Cluster;
 import org.apache.cassandra.distributed.api.IInstanceConfig;
 import org.apache.cassandra.distributed.api.TokenSupplier;
 import org.apache.cassandra.distributed.shared.NetworkTopology;
+import org.apache.cassandra.distributed.shared.WithProperties;
 import org.apache.cassandra.schema.Schema;
 
+import static org.apache.cassandra.config.CassandraRelevantProperties.BROADCAST_INTERVAL_MS;
 import static org.apache.cassandra.config.CassandraRelevantProperties.IGNORED_SCHEMA_CHECK_ENDPOINTS;
 import static org.apache.cassandra.config.CassandraRelevantProperties.IGNORED_SCHEMA_CHECK_VERSIONS;
+import static org.apache.cassandra.config.CassandraRelevantProperties.REPLACE_ADDRESS;
 import static org.apache.cassandra.config.CassandraRelevantProperties.RING_DELAY;
 import static org.apache.cassandra.distributed.api.Feature.GOSSIP;
 import static org.apache.cassandra.distributed.api.Feature.NETWORK;
 
 public class MigrationCoordinatorTest extends TestBaseImpl
 {
+    private static WithProperties withProperties;
+
     @Before
     public void setUp()
     {
-        // make the test a bit faster
-        RING_DELAY.setLong(5000);
-        System.setProperty("cassandra.broadcast_interval_ms", "30000");
+        withProperties = new WithProperties();
+        withProperties.set(RING_DELAY, 5000);
+        withProperties.set(BROADCAST_INTERVAL_MS, 30000);
     }
 
     @After
     public void afterTestCleanup()
     {
-        System.getProperties().remove("cassandra.replace_address");
-        IGNORED_SCHEMA_CHECK_VERSIONS.clearValue();
-        IGNORED_SCHEMA_CHECK_ENDPOINTS.clearValue();
-
-        RING_DELAY.clearValue();
-        System.getProperties().remove("cassandra.broadcast_interval_ms");
+        withProperties.close();
     }
 
     /**
@@ -78,7 +78,7 @@ public class MigrationCoordinatorTest extends TestBaseImpl
 
             IInstanceConfig config = cluster.newInstanceConfig();
             config.set("auto_bootstrap", true);
-            System.setProperty("cassandra.replace_address", replacementAddress.getHostAddress());
+            withProperties.set(REPLACE_ADDRESS, replacementAddress.getHostAddress());
             cluster.bootstrap(config).startup();
         }
     }
@@ -99,7 +99,7 @@ public class MigrationCoordinatorTest extends TestBaseImpl
 
             IInstanceConfig config = cluster.newInstanceConfig();
             config.set("auto_bootstrap", true);
-            IGNORED_SCHEMA_CHECK_ENDPOINTS.setString(ignoredEndpoint.getHostAddress());
+            withProperties.set(IGNORED_SCHEMA_CHECK_ENDPOINTS, ignoredEndpoint.getHostAddress());
             cluster.bootstrap(config).startup();
         }
     }
@@ -125,7 +125,7 @@ public class MigrationCoordinatorTest extends TestBaseImpl
 
             IInstanceConfig config = cluster.newInstanceConfig();
             config.set("auto_bootstrap", true);
-            IGNORED_SCHEMA_CHECK_VERSIONS.setString(initialVersion.toString() + ',' + oldVersion);
+            withProperties.set(IGNORED_SCHEMA_CHECK_VERSIONS, initialVersion.toString() + ',' + oldVersion);
             cluster.bootstrap(config).startup();
         }
     }
