@@ -20,10 +20,8 @@ package org.apache.cassandra.distributed.test;
 
 import java.io.IOException;
 import java.util.Collections;
-import java.util.EnumSet;
 import java.util.Iterator;
 import java.util.List;
-import java.util.Map;
 import java.util.concurrent.Callable;
 import java.util.concurrent.CyclicBarrier;
 import java.util.concurrent.TimeUnit;
@@ -31,14 +29,6 @@ import java.util.stream.IntStream;
 import java.util.stream.Stream;
 
 import com.google.common.util.concurrent.Uninterruptibles;
-import org.apache.cassandra.concurrent.SEPExecutor;
-import org.apache.cassandra.dht.Token;
-import org.apache.cassandra.locator.AbstractReplicationStrategy;
-import org.apache.cassandra.locator.EndpointsForToken;
-import org.apache.cassandra.locator.InetAddressAndPort;
-import org.apache.cassandra.locator.ReplicaLayout;
-import org.apache.cassandra.locator.ReplicaUtils;
-import org.apache.cassandra.utils.Throwables;
 import org.junit.Assert;
 import org.junit.Test;
 
@@ -46,21 +36,32 @@ import net.bytebuddy.ByteBuddy;
 import net.bytebuddy.dynamic.loading.ClassLoadingStrategy;
 import net.bytebuddy.implementation.MethodDelegation;
 import net.bytebuddy.implementation.bind.annotation.SuperCall;
-import org.apache.cassandra.db.*;
+import org.apache.cassandra.concurrent.SEPExecutor;
+import org.apache.cassandra.db.ColumnFamilyStore;
+import org.apache.cassandra.db.Keyspace;
+import org.apache.cassandra.db.ReadCommand;
+import org.apache.cassandra.db.ReadExecutionController;
+import org.apache.cassandra.db.SinglePartitionReadCommand;
 import org.apache.cassandra.db.partitions.UnfilteredPartitionIterator;
+import org.apache.cassandra.dht.Token;
 import org.apache.cassandra.distributed.Cluster;
 import org.apache.cassandra.distributed.api.ConsistencyLevel;
 import org.apache.cassandra.distributed.api.IInvokableInstance;
 import org.apache.cassandra.distributed.api.IIsolatedExecutor;
 import org.apache.cassandra.io.sstable.Descriptor;
 import org.apache.cassandra.io.sstable.format.SSTableReader;
-import org.apache.cassandra.io.sstable.metadata.MetadataComponent;
-import org.apache.cassandra.io.sstable.metadata.MetadataType;
+import org.apache.cassandra.io.sstable.format.StatsComponent;
 import org.apache.cassandra.io.sstable.metadata.StatsMetadata;
+import org.apache.cassandra.locator.AbstractReplicationStrategy;
+import org.apache.cassandra.locator.EndpointsForToken;
+import org.apache.cassandra.locator.InetAddressAndPort;
+import org.apache.cassandra.locator.ReplicaLayout;
+import org.apache.cassandra.locator.ReplicaUtils;
 import org.apache.cassandra.service.ActiveRepairService;
 import org.apache.cassandra.service.StorageProxy;
 import org.apache.cassandra.service.StorageProxy.LocalReadRunnable;
 import org.apache.cassandra.utils.DiagnosticSnapshotService;
+import org.apache.cassandra.utils.Throwables;
 
 import static net.bytebuddy.matcher.ElementMatchers.named;
 import static net.bytebuddy.matcher.ElementMatchers.takesArguments;
@@ -516,10 +517,7 @@ public class RepairDigestTrackingTest extends TestBaseImpl
                 {
                     SSTableReader sstable = sstables.next();
                     Descriptor descriptor = sstable.descriptor;
-                    Map<MetadataType, MetadataComponent> metadata = descriptor.getMetadataSerializer()
-                                                                              .deserialize(descriptor, EnumSet.of(MetadataType.STATS));
-
-                    StatsMetadata stats = (StatsMetadata) metadata.get(MetadataType.STATS);
+                    StatsMetadata stats = StatsComponent.load(descriptor).statsMetadata();
                     Assert.assertEquals("repaired at is set for sstable: " + descriptor,
                                         stats.repairedAt,
                                         ActiveRepairService.UNREPAIRED_SSTABLE);
@@ -568,10 +566,7 @@ public class RepairDigestTrackingTest extends TestBaseImpl
                 {
                     SSTableReader sstable = sstables.next();
                     Descriptor descriptor = sstable.descriptor;
-                    Map<MetadataType, MetadataComponent> metadata = descriptor.getMetadataSerializer()
-                                                                              .deserialize(descriptor, EnumSet.of(MetadataType.STATS));
-
-                    StatsMetadata stats = (StatsMetadata) metadata.get(MetadataType.STATS);
+                    StatsMetadata stats = StatsComponent.load(descriptor).statsMetadata();
                     Assert.assertTrue("repaired at is not set for sstable: " + descriptor, stats.repairedAt > 0);
                 }
             }
