@@ -71,7 +71,7 @@ public class ResourceLeakTest extends TestBaseImpl
     final boolean dumpEveryLoop = false;   // Dump heap & possibly files every loop
     final boolean dumpFileHandles = false; // Call lsof whenever dumping resources
     final boolean forceCollection = false; // Whether to explicitly force finalization/gc for smaller heap dumps
-    final long finalWaitMillis = 0l;       // Number of millis to wait before final resource dump to give gc a chance
+    final long finalWaitMillis = 0L;       // Number of millis to wait before final resource dump to give gc a chance
 
     static final SimpleDateFormat format = new SimpleDateFormat("yyyyMMddHHmmss");
     static final String when = format.format(Date.from(now()));
