@@ -18,86 +18,25 @@
 
 package org.apache.cassandra.cql3;
 
-import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.Collection;
 import java.util.List;
 import java.util.Map;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.CyclicBarrier;
-import java.util.stream.Collectors;
 
-import org.junit.After;
-import org.junit.Before;
-import org.junit.BeforeClass;
 import org.junit.Test;
-import org.junit.runner.RunWith;
-import org.junit.runners.Parameterized;
 
 import com.datastax.driver.core.Row;
 import com.datastax.driver.core.exceptions.NoHostAvailableException;
-import com.datastax.driver.core.exceptions.OperationTimedOutException;
 import com.datastax.driver.core.exceptions.WriteTimeoutException;
+import org.apache.cassandra.Util;
 import org.apache.cassandra.batchlog.BatchlogManager;
 import org.apache.cassandra.concurrent.NamedThreadFactory;
-import org.apache.cassandra.transport.ProtocolVersion;
-import org.apache.cassandra.concurrent.SEPExecutor;
-import org.apache.cassandra.concurrent.Stage;
 import org.apache.cassandra.db.Keyspace;
-import org.apache.cassandra.utils.FBUtilities;
 import org.apache.cassandra.utils.WrappedRunnable;
 
-@RunWith(Parameterized.class)
-public class ViewLongTest extends CQLTester
+public class ViewLongTest extends ViewAbstractParameterizedTest
 {
-    @Parameterized.Parameter
-    public ProtocolVersion version;
-
-    @Parameterized.Parameters()
-    public static Collection<Object[]> versions()
-    {
-        return ProtocolVersion.SUPPORTED.stream()
-                                        .map(v -> new Object[]{v})
-                                        .collect(Collectors.toList());
-    }
-
-    private final List<String> views = new ArrayList<>();
-
-    @BeforeClass
-    public static void startup()
-    {
-        requireNetwork();
-    }
-    @Before
-    public void begin()
-    {
-        views.clear();
-    }
-
-    @After
-    public void end() throws Throwable
-    {
-        for (String viewName : views)
-            executeNet(version, "DROP MATERIALIZED VIEW " + viewName);
-    }
-
-    private void createView(String name, String query) throws Throwable
-    {
-        try
-        {
-            executeNet(version, String.format(query, name));
-            // If exception is thrown, the view will not be added to the list; since it shouldn't have been created, this is
-            // the desired behavior
-            views.add(name);
-        }
-        catch (OperationTimedOutException ex)
-        {
-            // ... except for timeout, when we actually do not know whether the view was created or not
-            views.add(name);
-            throw ex;
-        }
-    }
-
     @Test
     public void testConflictResolution() throws Throwable
     {
@@ -111,9 +50,9 @@ public class ViewLongTest extends CQLTester
                     "c int," +
                     "PRIMARY KEY (a, b))");
 
-        executeNet(version, "USE " + keyspace());
-
-        createView("mv", "CREATE MATERIALIZED VIEW %s AS SELECT * FROM %%s WHERE c IS NOT NULL AND a IS NOT NULL AND b IS NOT NULL PRIMARY KEY (c, a, b)");
+        createView("CREATE MATERIALIZED VIEW %s AS SELECT * FROM %s " +
+                   "WHERE c IS NOT NULL AND a IS NOT NULL AND b IS NOT NULL " +
+                   "PRIMARY KEY (c, a, b)");
 
         CyclicBarrier semaphore = new CyclicBarrier(writers);
 
@@ -121,7 +60,7 @@ public class ViewLongTest extends CQLTester
         for (int i = 0; i < writers; i++)
         {
             final int writer = i;
-            Thread t = NamedThreadFactory.createThread(new WrappedRunnable()
+            Thread t = NamedThreadFactory.createAnonymousThread(new WrappedRunnable()
             {
                 public void runMayThrow()
                 {
@@ -133,7 +72,7 @@ public class ViewLongTest extends CQLTester
                         {
                             try
                             {
-                                executeNet(version, "INSERT INTO %s (a, b, c) VALUES (?, ?, ?) USING TIMESTAMP 1",
+                                executeNet("INSERT INTO %s (a, b, c) VALUES (?, ?, ?) USING TIMESTAMP 1",
                                            1,
                                            1,
                                            i + writerOffset);
@@ -159,7 +98,7 @@ public class ViewLongTest extends CQLTester
 
         for (int i = 0; i < writers * insertsPerWriter; i++)
         {
-            if (executeNet(version, "SELECT COUNT(*) FROM system.batches").one().getLong(0) == 0)
+            if (executeNet("SELECT COUNT(*) FROM system.batches").one().getLong(0) == 0)
                 break;
             try
             {
@@ -173,9 +112,9 @@ public class ViewLongTest extends CQLTester
             }
         }
 
-        int value = executeNet(version, "SELECT c FROM %s WHERE a = 1 AND b = 1").one().getInt("c");
+        int value = executeNet("SELECT c FROM %s WHERE a = 1 AND b = 1").one().getInt("c");
 
-        List<Row> rows = executeNet(version, "SELECT c FROM " + keyspace() + ".mv").all();
+        List<Row> rows = executeNet("SELECT c FROM " + keyspace() + "." + currentView()).all();
 
         boolean containsC = false;
         StringBuilder others = new StringBuilder();
@@ -203,13 +142,19 @@ public class ViewLongTest extends CQLTester
 
         if (rows.size() > 1)
         {
-            throw new AssertionError(String.format("Expected 1 row, but found %d; %s c = %d, and (%s) of which (%s) failed to insert", rows.size(), containsC ? "found row with" : "no rows contained", value, others, overlappingFailedWrites));
+            throw new AssertionError(String.format("Expected 1 row, but found %d; %s c = %d, " +
+                                                   "and (%s) of which (%s) failed to insert",
+                                                   rows.size(),
+                                                   containsC ? "found row with" : "no rows contained",
+                                                   value,
+                                                   others,
+                                                   overlappingFailedWrites));
         }
         else if (rows.isEmpty())
         {
             throw new AssertionError(String.format("Could not find row with c = %d", value));
         }
-        else if (rows.size() == 1 && !containsC)
+        else if (!containsC)
         {
             throw new AssertionError(String.format("Single row had c = %d, expected %d", rows.get(0).getInt("c"), value));
         }
@@ -231,104 +176,102 @@ public class ViewLongTest extends CQLTester
     {
         createTable("CREATE TABLE %s (field1 int,field2 int,date int,PRIMARY KEY ((field1), field2)) WITH default_time_to_live = 5;");
 
-        execute("USE " + keyspace());
-        executeNet(version, "USE " + keyspace());
-
-        createView("mv",
-                   "CREATE MATERIALIZED VIEW mv AS SELECT * FROM %%s WHERE field1 IS NOT NULL AND field2 IS NOT NULL AND date IS NOT NULL PRIMARY KEY ((field1), date, field2) WITH CLUSTERING ORDER BY (date desc, field2 asc);");
+        createView("CREATE MATERIALIZED VIEW %s AS SELECT * FROM %s " +
+                   "WHERE field1 IS NOT NULL AND field2 IS NOT NULL AND date IS NOT NULL " +
+                   "PRIMARY KEY ((field1), date, field2) WITH CLUSTERING ORDER BY (date desc, field2 asc)");
 
         updateViewWithFlush("insert into %s (field1, field2, date) values (1, 2, 111);", flush);
         assertRows(execute("select * from %s"), row(1, 2, 111));
-        assertRows(execute("select * from mv"), row(1, 111, 2));
+        assertRows(executeView("select * from %s"), row(1, 111, 2));
 
         updateViewWithFlush("insert into %s (field1, field2, date) values (1, 2, 222);", flush);
         assertRows(execute("select * from %s"), row(1, 2, 222));
-        assertRows(execute("select * from mv"), row(1, 222, 2));
+        assertRows(executeView("select * from %s"), row(1, 222, 2));
 
         updateViewWithFlush("insert into %s (field1, field2, date) values (1, 2, 333);", flush);
 
         assertRows(execute("select * from %s"), row(1, 2, 333));
-        assertRows(execute("select * from mv"), row(1, 333, 2));
+        assertRows(executeView("select * from %s"), row(1, 333, 2));
 
         if (flush)
         {
-            Keyspace.open(keyspace()).getColumnFamilyStore("mv").forceMajorCompaction();
+            Keyspace.open(keyspace()).getColumnFamilyStore(currentView()).forceMajorCompaction();
             assertRows(execute("select * from %s"), row(1, 2, 333));
-            assertRows(execute("select * from mv"), row(1, 333, 2));
+            assertRows(executeView("select * from %s"), row(1, 333, 2));
         }
 
         // wait for ttl, data should be removed
         updateViewWithFlush("insert into %s (field1, field2, date) values (1, 2, 444);", flush);
         assertRows(execute("select * from %s"), row(1, 2, 444));
-        assertRows(execute("select * from mv"), row(1, 444, 2));
+        assertRows(executeView("select * from %s"), row(1, 444, 2));
 
         Thread.sleep(5000);
         assertRows(execute("select * from %s"));
-        assertRows(execute("select * from mv"));
+        assertRows(executeView("select * from %s"));
 
         // shadow mv with date=555 and then update it back to live, wait for ttl
         updateView("update %s set date=555 where field1=1 and field2=2;");
         updateView("update %s set date=666 where field1=1 and field2=2;");
         updateViewWithFlush("update %s set date=555 where field1=1 and field2=2;", flush);
         assertRows(execute("select * from %s"), row(1, 2, 555));
-        assertRows(execute("select * from mv"), row(1, 555, 2));
+        assertRows(executeView("select * from %s"), row(1, 555, 2));
 
         Thread.sleep(5000);
         assertRows(execute("select * from %s"));
-        assertRows(execute("select * from mv"));
+        assertRows(executeView("select * from %s"));
 
         // test user-provided ttl for table with/without default-ttl
         for (boolean withDefaultTTL : Arrays.asList(true, false))
         {
             execute("TRUNCATE %s");
             if (withDefaultTTL)
-                execute("ALTER TABLE %s with default_time_to_live=" + (withDefaultTTL ? 10 : 0));
+                execute("ALTER TABLE %s with default_time_to_live=" + 10);
             updateViewWithFlush("insert into %s (field1, field2, date) values (1, 2, 666) USING TTL 1000;", flush);
 
             assertRows(execute("select * from %s"), row(1, 2, 666));
-            assertRows(execute("select * from mv"), row(1, 666, 2));
+            assertRows(executeView("select * from %s"), row(1, 666, 2));
 
             updateViewWithFlush("insert into %s (field1, field2, date) values (1, 2, 777) USING TTL 1100;", flush);
             assertRows(execute("select * from %s"), row(1, 2, 777));
-            assertRows(execute("select * from mv"), row(1, 777, 2));
+            assertRows(executeView("select * from %s"), row(1, 777, 2));
 
             updateViewWithFlush("insert into %s (field1, field2, date) values (1, 2, 888) USING TTL 800;", flush);
 
             assertRows(execute("select * from %s"), row(1, 2, 888));
-            assertRows(execute("select * from mv"), row(1, 888, 2));
+            assertRows(executeView("select * from %s"), row(1, 888, 2));
 
             if (flush)
             {
-                Keyspace.open(keyspace()).getColumnFamilyStore("mv").forceMajorCompaction();
+                Keyspace.open(keyspace()).getColumnFamilyStore(currentView()).forceMajorCompaction();
                 assertRows(execute("select * from %s"), row(1, 2, 888));
-                assertRows(execute("select * from mv"), row(1, 888, 2));
+                assertRows(executeView("select * from %s"), row(1, 888, 2));
             }
 
             // wait for ttl, data should be removed
             updateViewWithFlush("insert into %s (field1, field2, date) values (1, 2, 999) USING TTL 5;", flush);
             assertRows(execute("select * from %s"), row(1, 2, 999));
-            assertRows(execute("select * from mv"), row(1, 999, 2));
+            assertRows(executeView("select * from %s"), row(1, 999, 2));
 
             Thread.sleep(5000);
             assertRows(execute("select * from %s"));
-            assertRows(execute("select * from mv"));
+            assertRows(executeView("select * from %s"));
 
             // shadow mv with date=555 and then update it back to live with ttl=5, wait for ttl to expire
             updateViewWithFlush("update %s  USING TTL 800 set date=555 where field1=1 and field2=2;", flush);
             assertRows(execute("select * from %s"), row(1, 2, 555));
-            assertRows(execute("select * from mv"), row(1, 555, 2));
+            assertRows(executeView("select * from %s"), row(1, 555, 2));
 
             updateViewWithFlush("update %s set date=666 where field1=1 and field2=2;", flush);
             assertRows(execute("select * from %s"), row(1, 2, 666));
-            assertRows(execute("select * from mv"), row(1, 666, 2));
+            assertRows(executeView("select * from %s"), row(1, 666, 2));
 
             updateViewWithFlush("update %s USING TTL 5 set date=555 where field1=1 and field2=2;", flush);
             assertRows(execute("select * from %s"), row(1, 2, 555));
-            assertRows(execute("select * from mv"), row(1, 555, 2));
+            assertRows(executeView("select * from %s"), row(1, 555, 2));
 
             Thread.sleep(5000);
             assertRows(execute("select * from %s"));
-            assertRows(execute("select * from mv"));
+            assertRows(executeView("select * from %s"));
         }
     }
 
@@ -348,35 +291,32 @@ public class ViewLongTest extends CQLTester
     {
         createTable("CREATE TABLE %s (k int,c int,a int, b int, PRIMARY KEY ((k), c)) WITH default_time_to_live = 1000;");
 
-        execute("USE " + keyspace());
-        executeNet(version, "USE " + keyspace());
-
-        createView("mv",
-                   "CREATE MATERIALIZED VIEW mv AS SELECT k,c,a FROM %%s WHERE k IS NOT NULL AND c IS NOT NULL "
-                           + "PRIMARY KEY (c, k)");
+        createView("CREATE MATERIALIZED VIEW %s AS SELECT k,c,a FROM %s " +
+                   "WHERE k IS NOT NULL AND c IS NOT NULL " +
+                   "PRIMARY KEY (c, k)");
 
         // table default ttl
         updateViewWithFlush("UPDATE %s SET b = 111 WHERE k = 1 AND c = 2", flush);
         assertRows(execute("select k,c,a,b from %s"), row(1, 2, null, 111));
-        assertRows(execute("select k,c,a from mv"), row(1, 2, null));
+        assertRows(executeView("select k,c,a from %s"), row(1, 2, null));
 
         updateViewWithFlush("UPDATE %s SET b = null WHERE k = 1 AND c = 2", flush);
         assertRows(execute("select k,c,a,b from %s"));
-        assertRows(execute("select k,c,a from mv"));
+        assertRows(executeView("select k,c,a from %s"));
 
         updateViewWithFlush("UPDATE %s SET b = 222 WHERE k = 1 AND c = 2", flush);
         assertRows(execute("select k,c,a,b from %s"), row(1, 2, null, 222));
-        assertRows(execute("select k,c,a from mv"), row(1, 2, null));
+        assertRows(executeView("select k,c,a from %s"), row(1, 2, null));
 
         updateViewWithFlush("DELETE b FROM %s WHERE k = 1 AND c = 2", flush);
         assertRows(execute("select k,c,a,b from %s"));
-        assertRows(execute("select k,c,a from mv"));
+        assertRows(executeView("select k,c,a from %s"));
 
         if (flush)
         {
-            Keyspace.open(keyspace()).getColumnFamilyStore("mv").forceMajorCompaction();
+            Keyspace.open(keyspace()).getColumnFamilyStore(currentView()).forceMajorCompaction();
             assertRows(execute("select k,c,a,b from %s"));
-            assertRows(execute("select k,c,a from mv"));
+            assertRows(executeView("select k,c,a from %s"));
         }
 
         // test user-provided ttl for table with/without default-ttl
@@ -384,59 +324,42 @@ public class ViewLongTest extends CQLTester
         {
             execute("TRUNCATE %s");
             if (withDefaultTTL)
-                execute("ALTER TABLE %s with default_time_to_live=" + (withDefaultTTL ? 10 : 0));
+                execute("ALTER TABLE %s with default_time_to_live=" + 10);
 
             updateViewWithFlush("UPDATE %s USING TTL 100 SET b = 666 WHERE k = 1 AND c = 2", flush);
             assertRows(execute("select k,c,a,b from %s"), row(1, 2, null, 666));
-            assertRows(execute("select k,c,a from mv"), row(1, 2, null));
+            assertRows(executeView("select k,c,a from %s"), row(1, 2, null));
 
             updateViewWithFlush("UPDATE %s USING TTL 90  SET b = null WHERE k = 1 AND c = 2", flush);
             if (flush)
-                FBUtilities.waitOnFutures(Keyspace.open(keyspace()).flush());
+                Util.flushKeyspace(keyspace());
             assertRows(execute("select k,c,a,b from %s"));
-            assertRows(execute("select k,c,a from mv"));
+            assertRows(executeView("select k,c,a from %s"));
 
             updateViewWithFlush("UPDATE %s USING TTL 80  SET b = 777 WHERE k = 1 AND c = 2", flush);
             assertRows(execute("select k,c,a,b from %s"), row(1, 2, null, 777));
-            assertRows(execute("select k,c,a from mv"), row(1, 2, null));
+            assertRows(executeView("select k,c,a from %s"), row(1, 2, null));
 
             updateViewWithFlush("DELETE b FROM %s WHERE k = 1 AND c = 2", flush);
             assertRows(execute("select k,c,a,b from %s"));
-            assertRows(execute("select k,c,a from mv"));
+            assertRows(executeView("select k,c,a from %s"));
 
             updateViewWithFlush("UPDATE %s USING TTL 110  SET b = 888 WHERE k = 1 AND c = 2", flush);
             assertRows(execute("select k,c,a,b from %s"), row(1, 2, null, 888));
-            assertRows(execute("select k,c,a from mv"), row(1, 2, null));
+            assertRows(executeView("select k,c,a from %s"), row(1, 2, null));
 
             updateViewWithFlush("UPDATE %s USING TTL 5  SET b = 999 WHERE k = 1 AND c = 2", flush);
             assertRows(execute("select k,c,a,b from %s"), row(1, 2, null, 999));
-            assertRows(execute("select k,c,a from mv"), row(1, 2, null));
+            assertRows(executeView("select k,c,a from %s"), row(1, 2, null));
 
             Thread.sleep(5000); // wait for ttl expired
 
             if (flush)
             {
-                Keyspace.open(keyspace()).getColumnFamilyStore("mv").forceMajorCompaction();
+                Keyspace.open(keyspace()).getColumnFamilyStore(currentView()).forceMajorCompaction();
                 assertRows(execute("select k,c,a,b from %s"));
-                assertRows(execute("select k,c,a from mv"));
+                assertRows(executeView("select k,c,a from %s"));
             }
         }
     }
-
-    private void updateView(String query, Object... params) throws Throwable
-    {
-        updateViewWithFlush(query, false, params);
-    }
-
-    private void updateViewWithFlush(String query, boolean flush, Object... params) throws Throwable
-    {
-        executeNet(version, query, params);
-        while (!(((SEPExecutor) Stage.VIEW_MUTATION.executor()).getPendingTaskCount() == 0
-                && ((SEPExecutor) Stage.VIEW_MUTATION.executor()).getActiveTaskCount() == 0))
-        {
-            Thread.sleep(1);
-        }
-        if (flush)
-            Keyspace.open(keyspace()).flush();
-    }
 }
