@@ -80,11 +80,21 @@ public abstract class ReadBenchBase extends SimpleTableWriter
             // don't flush
         }
 
+        System.err.format("%s sstables, total %s, %,d partitions. Mean write latency %.2f ms\n",
+                          cfs.getLiveSSTables().size(),
+                          FBUtilities.prettyPrintMemory(cfs.metric.liveDiskSpaceUsed.getCount()),
+                          cfs.metric.estimatedPartitionCount.getValue(),
+                          cfs.metric.writeLatency.latency.getSnapshot().getMean());
         // Needed to stabilize sstable count for off-cache sized tests (e.g. count = 100_000_000)
         while (cfs.getLiveSSTables().size() >= 15)
         {
             cfs.enableAutoCompaction(true);
             cfs.disableAutoCompaction();
+            System.err.format("%s sstables, total %s, %,d partitions. Mean write latency %.2f ms\n",
+                              cfs.getLiveSSTables().size(),
+                              FBUtilities.prettyPrintMemory(cfs.metric.liveDiskSpaceUsed.getCount()),
+                              cfs.metric.estimatedPartitionCount.getValue(),
+                              cfs.metric.writeLatency.latency.getSnapshot().getMean());
         }
     }
 
@@ -109,7 +119,8 @@ public abstract class ReadBenchBase extends SimpleTableWriter
                                                    }
                                                    catch (Throwable throwable)
                                                    {
-                                                       throw Throwables.propagate(throwable);
+                                                       Throwables.throwIfUnchecked(throwable);
+                                                       throw new RuntimeException(throwable);
                                                    }
                                                }));
         }
@@ -142,7 +153,8 @@ public abstract class ReadBenchBase extends SimpleTableWriter
                                                    }
                                                    catch (Throwable throwable)
                                                    {
-                                                       throw Throwables.propagate(throwable);
+                                                       Throwables.throwIfUnchecked(throwable);
+                                                       throw new RuntimeException(throwable);
                                                    }
                                                }));
         }
