@@ -22,6 +22,8 @@ import java.util.function.Consumer;
 
 import org.apache.cassandra.exceptions.ConfigurationException;
 
+import static org.apache.cassandra.config.CassandraRelevantProperties.CONFIG_LOADER;
+
 /**
  * Helper class for programmatically overriding individual config values before DatabaseDescriptor is bootstrapped.
  */
@@ -40,7 +42,7 @@ public class OverrideConfigurationLoader implements ConfigurationLoader
 
     public static void override(Consumer<Config> modifier)
     {
-        System.setProperty(Config.PROPERTY_PREFIX + "config.loader", OverrideConfigurationLoader.class.getName());
+        CONFIG_LOADER.setString(OverrideConfigurationLoader.class.getName());
         configModifier = modifier;
     }
 }
