@@ -17,11 +17,16 @@
  */
 package org.apache.cassandra.cql3.validation.entities;
 
+import java.util.Arrays;
+import java.util.List;
+
 import org.junit.Test;
 
 import org.junit.Assert;
 import org.apache.cassandra.cql3.CQLTester;
+import org.apache.cassandra.utils.Pair;
 
+import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertNull;
 import static org.junit.Assert.assertTrue;
 
@@ -66,7 +71,7 @@ public class TimestampTest extends CQLTester
 
 
         // wrap writetime(), ttl() in other functions (test for CASSANDRA-8451)
-        res = getRows(execute("SELECT k, c, blobAsBigint(bigintAsBlob(writetime(c))), ttl(c) FROM %s"));
+        res = getRows(execute("SELECT k, c, blob_as_bigint(bigint_as_blob(writetime(c))), ttl(c) FROM %s"));
         Assert.assertEquals(2, res.length);
 
         for (Object[] r : res)
@@ -78,7 +83,7 @@ public class TimestampTest extends CQLTester
                 assertTrue(r[3] instanceof Integer || r[2] instanceof Long);
         }
 
-        res = getRows(execute("SELECT k, c, writetime(c), blobAsInt(intAsBlob(ttl(c))) FROM %s"));
+        res = getRows(execute("SELECT k, c, writetime(c), blob_as_int(int_as_blob(ttl(c))) FROM %s"));
         Assert.assertEquals(2, res.length);
 
 
@@ -97,6 +102,84 @@ public class TimestampTest extends CQLTester
                    row(1, null, null));
     }
 
+    private void setupSchemaForMaxTimestamp()
+    {
+        String myType = createType("CREATE TYPE %s (a int, b int)");
+        createTable("CREATE TABLE %s (k int PRIMARY KEY, a text, " +
+                    "l list<int>, fl frozen<list<int>>," +
+                    "s set<int>, fs frozen<set<int>>," +
+                    "m map<int, text>, fm frozen<map<int, text>>," +
+                    "t " + myType + ", ft frozen<" + myType + ">)");
+    }
+
+    @Test
+    public void testCallMaxTimestampOnEmptyCollectionReturnsNull() throws Throwable
+    {
+        setupSchemaForMaxTimestamp();
+
+        execute("INSERT INTO %s (k) VALUES (1)");
+        Object[][] res = getRows(execute("SELECT maxwritetime(a), maxwritetime(l), maxwritetime(fl)," +
+                                         "maxwritetime(s), maxwritetime(fs), maxwritetime(m), maxwritetime(fm)," +
+                                         "maxwritetime(t), maxwritetime(ft) FROM %s WHERE k=1"));
+
+        assertEquals(1, res.length);
+        for (Object v : res[0])
+        {
+            assertNull("All the multi-cell data are empty (we did not insert), calling maxwritetime should return null",
+                       v);
+        }
+    }
+
+    @Test
+    public void testMaxTimestamp() throws Throwable
+    {
+        setupSchemaForMaxTimestamp();
+
+        execute("INSERT INTO %s (k, a, l, fl, s, fs, m, fm, t, ft) VALUES " +
+                "(1, 'test', [1], [2], {1}, {2}, {1 : 'a'}, {2 : 'b'}, {a : 1, b : 1 }, {a : 2, b : 2}) USING TIMESTAMP 1");
+
+        // enumerate through all multi-cell types and make sure maxwritetime reflects the expected result
+        testMaxTimestampWithColumnUpdate(Arrays.asList(
+           Pair.create(1, "UPDATE %s USING TIMESTAMP 10 SET l = l + [10] WHERE k = 1"),
+           Pair.create(3, "UPDATE %s USING TIMESTAMP 11 SET s = s + {10} WHERE k = 1"),
+           Pair.create(5, "UPDATE %s USING TIMESTAMP 12 SET m = m + {10 : 'c'} WHERE k = 1"),
+           Pair.create(7, "UPDATE %s USING TIMESTAMP 13 SET t.a = 10 WHERE k = 1")
+        ));
+    }
+
+    private void testMaxTimestampWithColumnUpdate(List<Pair<Integer, String>> updateStatements) throws Throwable
+    {
+        for (Pair<Integer, String> update : updateStatements)
+        {
+            int fieldPos = update.left();
+            String statement = update.right();
+
+            // run the update statement and update the timestamp of the column
+            execute(statement);
+
+            Object[][] res = getRows(execute("SELECT maxwritetime(a), maxwritetime(l), maxwritetime(fl)," +
+                                             "maxwritetime(s), maxwritetime(fs), maxwritetime(m), maxwritetime(fm)," +
+                                             "maxwritetime(t), maxwritetime(ft) FROM %s WHERE k=1"));
+            Assert.assertEquals(1, res.length);
+            Assert.assertEquals("maxwritetime should work on both single cell and complex columns",
+                                9, res[0].length);
+            for (Object ts : res[0])
+            {
+                assertTrue(ts instanceof Long); // all the result fields are timestamps
+            }
+
+            long updatedTs = (long) res[0][fieldPos]; // maxwritetime the updated column
+
+            for (int i = 0; i < res[0].length; i++)
+            {
+                long ts = (long) res[0][i];
+                if (i != fieldPos)
+                    assertTrue("The updated column should have a large maxwritetime since it is updated later",
+                               ts < updatedTs);
+            }
+        }
+    }
+
     /**
      * Migrated from cql_tests.py:TestCQL.invalid_custom_timestamp_test()
      */
