@@ -25,14 +25,15 @@ import java.util.Arrays;
 import org.junit.BeforeClass;
 import org.junit.Test;
 
-import static junit.framework.Assert.assertNull;
-import static org.junit.Assert.assertEquals;
-
+import org.apache.cassandra.Util;
 import org.apache.cassandra.config.DatabaseDescriptor;
 import org.apache.cassandra.cql3.CQLTester;
 import org.apache.cassandra.cql3.restrictions.StatementRestrictions;
 import org.apache.cassandra.dht.ByteOrderedPartitioner;
 
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertNull;
+
 /**
  * SELECT statement tests that require a ByteOrderedPartitioner
  */
@@ -47,6 +48,7 @@ public class SelectOrderedPartitionerTest extends CQLTester
     @Test
     public void testTokenAndIndex() throws Throwable
     {
+        Util.assumeLegacySecondaryIndex();
         createTable("CREATE TABLE %s (a int, b int, c int, d int, PRIMARY KEY (a, b, c))");
         createIndex("CREATE INDEX ON %s(c)");
 
@@ -86,6 +88,7 @@ public class SelectOrderedPartitionerTest extends CQLTester
     @Test
     public void testFilteringOnPartitionKeyWithToken() throws Throwable
     {
+        Util.assumeLegacySecondaryIndex();
         createTable("CREATE TABLE %s (a int, b int, c int, d int, PRIMARY KEY ((a, b), c))");
         createIndex("CREATE INDEX ON %s(d)");
 
@@ -316,6 +319,7 @@ public class SelectOrderedPartitionerTest extends CQLTester
     @Test
     public void testMultiColumnPartitionKeyWithIndexAndTokenNonTokenRestrictionsMix() throws Throwable
     {
+        Util.assumeLegacySecondaryIndex();
         createTable("CREATE TABLE %s (a int, b int, c int, primary key((a, b)))");
         createIndex("CREATE INDEX ON %s(b)");
         createIndex("CREATE INDEX ON %s(c)");
@@ -368,7 +372,7 @@ public class SelectOrderedPartitionerTest extends CQLTester
 
         Object[][] rows = getRows(execute("SELECT v2 FROM %s"));
         assertEquals(0, rows[0][0]);
-        assertEquals(null, rows[1][0]);
+        assertNull(rows[1][0]);
         assertEquals(2, rows[2][0]);
 
         rows = getRows(execute("SELECT v2 FROM %s WHERE k = 1"));
@@ -402,6 +406,7 @@ public class SelectOrderedPartitionerTest extends CQLTester
     @Test
     public void testCompositeIndexWithPK() throws Throwable
     {
+        Util.assumeLegacySecondaryIndex();
         createTable("CREATE TABLE %s (blog_id int, time1 int, time2 int, author text, content text, PRIMARY KEY (blog_id, time1, time2))");
 
         createIndex("CREATE INDEX ON %s(author)");
@@ -533,6 +538,7 @@ public class SelectOrderedPartitionerTest extends CQLTester
     @Test
     public void testIndexOnCompositeWithCollections() throws Throwable
     {
+        Util.assumeLegacySecondaryIndex();
         createTable("CREATE TABLE %s (blog_id int, time1 int, time2 int, author text, content set<text>, PRIMARY KEY (blog_id, time1, time2))");
 
         createIndex("CREATE INDEX ON %s (author)");
