@@ -21,11 +21,12 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.List;
 
+import org.junit.Test;
+
+import org.apache.cassandra.Util;
 import org.apache.cassandra.cql3.CQLTester;
 import org.apache.cassandra.cql3.restrictions.StatementRestrictions;
 
-import org.junit.Test;
-
 public class SelectSingleColumnRelationTest extends CQLTester
 {
     @Test
@@ -456,6 +457,8 @@ public class SelectSingleColumnRelationTest extends CQLTester
     @Test
     public void testMultiplePartitionKeyWithIndex() throws Throwable
     {
+        Util.assumeLegacySecondaryIndex(); // SAI does not allow multi-column slice restrictions
+
         createTable("CREATE TABLE %s (a int, b int, c int, d int, e int, f int, PRIMARY KEY ((a, b), c, d, e))");
         createIndex("CREATE INDEX ON %s (c)");
         createIndex("CREATE INDEX ON %s (f)");
@@ -541,8 +544,8 @@ public class SelectSingleColumnRelationTest extends CQLTester
 
         assertInvalidMessage("Invalid unset value for argument in call to function token",
                              "SELECT * FROM %s WHERE token(k) >= token(?)", unset());
-        assertInvalidMessage("Invalid unset value for argument in call to function blobasint",
-                             "SELECT * FROM %s WHERE k = blobAsInt(?)", unset());
+        assertInvalidMessage("Invalid unset value for argument in call to function blob_as_int",
+                             "SELECT * FROM %s WHERE k = blob_as_int(?)", unset());
     }
 
     @Test
