@@ -159,7 +159,7 @@ public class ThrottledUnfilteredIteratorTest extends CQLTester
             UnfilteredRowIterator iterator = throttled.next();
             assertFalse(throttled.hasNext());
             assertFalse(iterator.hasNext());
-            assertEquals(Int32Type.instance.getSerializer().deserialize(iterator.staticRow().cells().iterator().next().buffer()), new Integer(160));
+            assertEquals(Int32Type.instance.getSerializer().deserialize(iterator.staticRow().cells().iterator().next().buffer()), Integer.valueOf(160));
         }
 
         // test opt out
@@ -366,7 +366,7 @@ public class ThrottledUnfilteredIteratorTest extends CQLTester
         {
             origin = rows(metadata.regularAndStaticColumns(),
                           1,
-                          new DeletionTime(0, 100),
+                          DeletionTime.build(0, 100),
                           createStaticRow(createCell(staticMetadata, 160)),
                           rows.toArray(new Row[0]));
             throttledIterator = new ThrottledUnfilteredIterator(origin, throttle);
@@ -413,7 +413,7 @@ public class ThrottledUnfilteredIteratorTest extends CQLTester
         for (int throttle = 2; throttle < 1200; throttle += 21)
         {
             origin = partitions(metadata.regularAndStaticColumns(),
-                                new DeletionTime(0, 100),
+                                DeletionTime.build(0, 100),
                                 createStaticRow(createCell(staticMetadata, 160)),
                                 partitions);
             throttledIterator = ThrottledUnfilteredIterator.throttle(origin, throttle);
@@ -434,7 +434,7 @@ public class ThrottledUnfilteredIteratorTest extends CQLTester
                 }
                 UnfilteredRowIterator current = rows(metadata.regularAndStaticColumns(),
                                                      currentPartition,
-                                                     new DeletionTime(0, 100),
+                                                     DeletionTime.build(0, 100),
                                                      createStaticRow(createCell(staticMetadata, 160)),
                                                      partitions.get(currentPartition).toArray(new Row[0]));
                 assertMetadata(current, splitted, currentSplit == 1);
@@ -450,7 +450,7 @@ public class ThrottledUnfilteredIteratorTest extends CQLTester
 
 
         origin = partitions(metadata.regularAndStaticColumns(),
-                            new DeletionTime(0, 100),
+                            DeletionTime.build(0, 100),
                             Rows.EMPTY_STATIC_ROW,
                             partitions);
         try
@@ -593,7 +593,7 @@ public class ThrottledUnfilteredIteratorTest extends CQLTester
         return createCell(metadata, v, 100L, BufferCell.NO_DELETION_TIME);
     }
 
-    private static Cell<?> createCell(ColumnMetadata metadata, int v, long timestamp, int localDeletionTime)
+    private static Cell<?> createCell(ColumnMetadata metadata, int v, long timestamp, long localDeletionTime)
     {
         return new BufferCell(metadata,
                               timestamp,
@@ -682,7 +682,7 @@ public class ThrottledUnfilteredIteratorTest extends CQLTester
             // Verify throttled data after merge
             Partition partition = ImmutableBTreePartition.create(UnfilteredRowIterators.merge(unfilteredRowIterators));
 
-            int nowInSec = FBUtilities.nowInSeconds();
+            long nowInSec = FBUtilities.nowInSeconds();
 
             for (int i : live)
                 assertTrue("Row " + i + " should be live", partition.getRow(Clustering.make(ByteBufferUtil.bytes((i)))).hasLiveData(nowInSec, cfs.metadata().enforceStrictLiveness()));
