@@ -35,7 +35,9 @@ import io.netty.handler.ssl.SslContext;
 import io.netty.handler.ssl.SslProvider;
 import org.apache.cassandra.config.EncryptionOptions;
 import org.apache.cassandra.config.ParameterizedClass;
+import org.apache.cassandra.distributed.shared.WithProperties;
 
+import static org.apache.cassandra.config.CassandraRelevantProperties.DISABLE_TCACTIVE_OPENSSL;
 import static org.apache.cassandra.security.PEMBasedSslContextFactory.ConfigKey.ENCODED_CERTIFICATES;
 import static org.apache.cassandra.security.PEMBasedSslContextFactory.ConfigKey.ENCODED_KEY;
 import static org.apache.cassandra.security.PEMBasedSslContextFactory.ConfigKey.KEY_PASSWORD;
@@ -216,6 +218,27 @@ public class PEMBasedSslContextFactoryTest
                                                            .withRequireClientAuth(false)
                                                            .withCipherSuites("TLS_RSA_WITH_AES_128_CBC_SHA")
                                                            .withSslContextFactory(sslContextFactory);
+        SslContext sslContext = SSLFactory.getOrCreateSslContext(options, true, ISslContextFactory.SocketType.SERVER, "test");
+        Assert.assertNotNull(sslContext);
+        if (OpenSsl.isAvailable())
+            Assert.assertTrue(sslContext instanceof OpenSslContext);
+        else
+            Assert.assertTrue(sslContext instanceof SslContext);
+    }
+
+    @Test
+    public void getSslContextOpenSSLOutboundKeystore() throws IOException
+    {
+        ParameterizedClass sslContextFactory = new ParameterizedClass(PEMBasedSslContextFactory.class.getSimpleName()
+        , new HashMap<>());
+        EncryptionOptions.ServerEncryptionOptions options = new EncryptionOptions.ServerEncryptionOptions().withTrustStore("test/conf/cassandra_ssl_test.truststore.pem")
+                                                                                                           .withKeyStore("test/conf/cassandra_ssl_test.keystore.pem")
+                                                                                                           .withKeyStorePassword("cassandra")
+                                                                                                           .withOutboundKeystore("test/conf/cassandra_ssl_test.keystore.pem")
+                                                                                                           .withOutboundKeystorePassword("cassandra")
+                                                                                                           .withRequireClientAuth(false)
+                                                                                                           .withCipherSuites("TLS_RSA_WITH_AES_128_CBC_SHA")
+                                                                                                           .withSslContextFactory(sslContextFactory);
         SslContext sslContext = SSLFactory.getOrCreateSslContext(options, true, ISslContextFactory.SocketType.CLIENT, "test");
         Assert.assertNotNull(sslContext);
         if (OpenSsl.isAvailable())
@@ -233,7 +256,7 @@ public class PEMBasedSslContextFactoryTest
         config.put("truststore", "/this/is/probably/not/a/file/on/your/test/machine");
 
         DefaultSslContextFactory defaultSslContextFactoryImpl = new DefaultSslContextFactory(config);
-        defaultSslContextFactoryImpl.checkedExpiry = false;
+        defaultSslContextFactoryImpl.keystoreContext.checkedExpiry = false;
         defaultSslContextFactoryImpl.buildTrustManagerFactory();
     }
 
@@ -244,7 +267,7 @@ public class PEMBasedSslContextFactoryTest
         config.putAll(commonConfig);
 
         PEMBasedSslContextFactory sslContextFactory = new PEMBasedSslContextFactory(config);
-        sslContextFactory.checkedExpiry = false;
+        sslContextFactory.keystoreContext.checkedExpiry = false;
         TrustManagerFactory trustManagerFactory = sslContextFactory.buildTrustManagerFactory();
         Assert.assertNotNull(trustManagerFactory);
     }
@@ -258,7 +281,7 @@ public class PEMBasedSslContextFactoryTest
         addFileBaseTrustStoreOptions(config);
 
         PEMBasedSslContextFactory sslContextFactory = new PEMBasedSslContextFactory(config);
-        sslContextFactory.checkedExpiry = false;
+        sslContextFactory.keystoreContext.checkedExpiry = false;
         TrustManagerFactory trustManagerFactory = sslContextFactory.buildTrustManagerFactory();
         Assert.assertNotNull(trustManagerFactory);
     }
@@ -271,7 +294,7 @@ public class PEMBasedSslContextFactoryTest
         config.put("keystore", "/this/is/probably/not/a/file/on/your/test/machine");
 
         PEMBasedSslContextFactory sslContextFactory = new PEMBasedSslContextFactory(config);
-        sslContextFactory.checkedExpiry = false;
+        sslContextFactory.keystoreContext.checkedExpiry = false;
         sslContextFactory.buildKeyManagerFactory();
     }
 
@@ -295,20 +318,20 @@ public class PEMBasedSslContextFactoryTest
 
         PEMBasedSslContextFactory sslContextFactory1 = new PEMBasedSslContextFactory(config);
         // Make sure the exiry check didn't happen so far for the private key
-        Assert.assertFalse(sslContextFactory1.checkedExpiry);
+        Assert.assertFalse(sslContextFactory1.keystoreContext.checkedExpiry);
 
         addKeyStoreOptions(config);
         PEMBasedSslContextFactory sslContextFactory2 = new PEMBasedSslContextFactory(config);
         // Trigger the private key loading. That will also check for expired private key
         sslContextFactory2.buildKeyManagerFactory();
         // Now we should have checked the private key's expiry
-        Assert.assertTrue(sslContextFactory2.checkedExpiry);
+        Assert.assertTrue(sslContextFactory2.keystoreContext.checkedExpiry);
 
         // Make sure that new factory object preforms the fresh private key expiry check
         PEMBasedSslContextFactory sslContextFactory3 = new PEMBasedSslContextFactory(config);
-        Assert.assertFalse(sslContextFactory3.checkedExpiry);
+        Assert.assertFalse(sslContextFactory3.keystoreContext.checkedExpiry);
         sslContextFactory3.buildKeyManagerFactory();
-        Assert.assertTrue(sslContextFactory3.checkedExpiry);
+        Assert.assertTrue(sslContextFactory3.keystoreContext.checkedExpiry);
     }
 
     @Test(expected = IllegalArgumentException.class)
@@ -343,20 +366,20 @@ public class PEMBasedSslContextFactoryTest
 
         PEMBasedSslContextFactory sslContextFactory1 = new PEMBasedSslContextFactory(config);
         // Make sure the expiry check didn't happen so far for the private key
-        Assert.assertFalse(sslContextFactory1.checkedExpiry);
+        Assert.assertFalse(sslContextFactory1.keystoreContext.checkedExpiry);
 
         addFileBaseKeyStoreOptions(config);
         PEMBasedSslContextFactory sslContextFactory2 = new PEMBasedSslContextFactory(config);
         // Trigger the private key loading. That will also check for expired private key
         sslContextFactory2.buildKeyManagerFactory();
         // Now we should have checked the private key's expiry
-        Assert.assertTrue(sslContextFactory2.checkedExpiry);
+        Assert.assertTrue(sslContextFactory2.keystoreContext.checkedExpiry);
 
         // Make sure that new factory object preforms the fresh private key expiry check
         PEMBasedSslContextFactory sslContextFactory3 = new PEMBasedSslContextFactory(config);
-        Assert.assertFalse(sslContextFactory3.checkedExpiry);
+        Assert.assertFalse(sslContextFactory3.keystoreContext.checkedExpiry);
         sslContextFactory3.buildKeyManagerFactory();
-        Assert.assertTrue(sslContextFactory3.checkedExpiry);
+        Assert.assertTrue(sslContextFactory3.keystoreContext.checkedExpiry);
     }
 
     @Test
@@ -392,13 +415,14 @@ public class PEMBasedSslContextFactoryTest
     {
         // The configuration name below is hard-coded intentionally to make sure we don't break the contract without
         // changing the documentation appropriately
-        System.setProperty("cassandra.disable_tcactive_openssl", "true");
-        Map<String, Object> config = new HashMap<>();
-        config.putAll(commonConfig);
+        try (WithProperties properties = new WithProperties().set(DISABLE_TCACTIVE_OPENSSL, true))
+        {
+            Map<String, Object> config = new HashMap<>();
+            config.putAll(commonConfig);
 
-        PEMBasedSslContextFactory sslContextFactory = new PEMBasedSslContextFactory(config);
-        Assert.assertEquals(SslProvider.JDK, sslContextFactory.getSslProvider());
-        System.clearProperty("cassandra.disable_tcactive_openssl");
+            PEMBasedSslContextFactory sslContextFactory = new PEMBasedSslContextFactory(config);
+            Assert.assertEquals(SslProvider.JDK, sslContextFactory.getSslProvider());
+        }
     }
 
     @Test(expected = IllegalArgumentException.class)
