@@ -28,6 +28,7 @@ import org.junit.Test;
 import com.fasterxml.jackson.core.type.TypeReference;
 import com.fasterxml.jackson.databind.ObjectMapper;
 import com.fasterxml.jackson.databind.exc.MismatchedInputException;
+import org.apache.cassandra.utils.JsonUtils;
 import org.assertj.core.api.Assertions;
 
 import static org.hamcrest.CoreMatchers.startsWith;
@@ -44,7 +45,7 @@ import static org.junit.Assert.fail;
  */
 public class SSTableExportSchemaLoadingTest extends OfflineToolUtils
 {
-    private static final ObjectMapper mapper = new ObjectMapper();
+    private static final ObjectMapper mapper = JsonUtils.JSON_OBJECT_MAPPER;
     private static final TypeReference<List<Map<String, Object>>> jacksonListOfMapsType = new TypeReference<List<Map<String, Object>>>() {};
     private static String sstable;
 
@@ -182,7 +183,7 @@ public class SSTableExportSchemaLoadingTest extends OfflineToolUtils
      */
     private void assertPostTestEnv()
     {
-        assertNoUnexpectedThreadsStarted(OPTIONAL_THREADS_WITH_SCHEMA);
+        assertNoUnexpectedThreadsStarted(OPTIONAL_THREADS_WITH_SCHEMA, true);
         assertCLSMNotLoaded();
         assertSystemKSNotLoaded();
         assertKeyspaceNotLoaded();
