@@ -33,12 +33,20 @@ import org.apache.cassandra.config.DatabaseDescriptor;
 import org.apache.cassandra.metrics.ClientMetrics;
 import org.apache.cassandra.service.QueryState;
 import org.apache.cassandra.transport.messages.AuthResponse;
+import org.mockito.Mockito;
 
 public class MessageDispatcherTest
 {
     static final Message.Request AUTH_RESPONSE_REQUEST = new AuthResponse(new byte[0])
     {
-        public Response execute(QueryState queryState, long queryStartNanoTime, boolean traceRequest)
+        @Override
+        public Connection connection()
+        {
+            return connectionMock();
+        }
+
+        @Override
+        public Response execute(QueryState queryState, Dispatcher.RequestTime requestTime, boolean traceRequest)
         {
             return null;
         }
@@ -62,14 +70,6 @@ public class MessageDispatcherTest
         DatabaseDescriptor.setNativeTransportMaxAuthThreads(maxAuthThreadsBeforeTests);
     }
 
-    @Test
-    public void testAuthRateLimiterTurnedOffByDefault() throws Exception
-    {
-        // All requests were executed on main request executor
-        Assert.assertEquals(0, tryAuth(this::completedAuth));
-        Assert.assertEquals(1, completedRequests());
-    }
-
     @Test
     public void testAuthRateLimiter() throws Exception
     {
@@ -99,6 +99,13 @@ public class MessageDispatcherTest
             long auths = completedAuth();
             long requests = tryAuth(this::completedRequests, new Message.Request(type)
             {
+                @Override
+                public Connection connection()
+                {
+                    return connectionMock();
+                }
+
+                @Override
                 public Response execute(QueryState queryState, Dispatcher.RequestTime requestTime, boolean traceRequest)
                 {
                     return null;
@@ -167,8 +174,6 @@ public class MessageDispatcherTest
             super(false);
         }
 
-
-
         @Override
         void processRequest(Channel channel,
                             Message.Request request,
@@ -179,4 +184,13 @@ public class MessageDispatcherTest
             // noop
         }
     }
+
+    private static Connection connectionMock()
+    {
+        Connection.Tracker tracker = Mockito.mock(Connection.Tracker.class);
+        Mockito.when(tracker.isRunning()).thenAnswer(invocation -> true);
+        Connection c = Mockito.mock(Connection.class);
+        Mockito.when(c.getTracker()).thenAnswer(invocation -> tracker);
+        return c;
+    }
 }
